C**********************************************************************
C                 Copyright 1992-1996, Robert Fraczkiewicz            *
C                           All rights reserved.                      *
C**********************************************************************
C This program is free software; you can redistribute it and/or modify*
C it under the terms of the GNU General Public License as published by*
C the Free Software Foundation; either version 1, or (at your option) *
C any later version.                                                  *
C                                                                     *
C This program is distributed in the hope that it will be useful,     *
C but WITHOUT ANY WARRANTY; without even the implied warranty of      *
C MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU    *
C General Public License for more details.                            *
C                                                                     *
C You should  have received a copy of the GNU General Public License  *
C along with this program; if not, write to the Free Software         *
C Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.           *
c**********************************************************************
c                                                                     *
c     Program "Displacement Coordinates" by Robert Fraczkiewicz       *
c                        Nov 1, 94, version 2.0                       *
c                                                                     *
c**********************************************************************
c The program calculates atomic internal and Cartesian displacements  *
c (commonly known as "eigenvectors") from normal mode displacements.  *
c The rectangular pseudoinverse matrix used to relate coordinate sets *
c is consistent with Eckart-Sayvetz conditions (Gussoni, M.;          *
c Dellepiane, G.; Abbate, S. J.Mol.Spectr. 1975,57,323-330).          *
c The secular equation is solved independently from program FR.       *
c Symmetry blocks in the ".rein" file must be arranged according to   *
c the molecule number. The total number of blocks must be equal to    *
c that in the ".umat" file.                                           *
c**********************************************************************
c                                                                     *
c     INPUT FILES:                                                    *
c                                                                     *
c     ".rein" - main input file containing force field and obseved    *
c               vibrational frequencies                               *
c     ".umat" - U-matrix (user-defined)                               *
c     ".zmat" - Z-matrix (output of ZSYM4)                            *
c     ".bmat" - B-matrix (output of GMAT4)                            *
c     ".gmat" - G-matrix (output of GMAT4)                            *
c     ".beta" - beta-matrix (Eckert-Sayvetz conditions, output of     *
c               GMAT4)                                                *
c     ".xmat" - X-matrix (output of GEOM)                             *
c                                                                     *
c**********************************************************************
c                                                                     *
c     OUTPUT FILES:                                                   *
c                                                                     *
c     ".int" - internal coordinate displacements (direct values)      *
c     ".xyz" - Cartesian displacements in a format readable by XMol   *
c              (use Animate option to see all modes)                  *
c                                                                     *
c**********************************************************************
c                                                                     *
c     BLAS AND LAPACK ROUTINES:                                       *
c                                                                     *
c     dsymm - symmetric matrix product                                *
c     dgemm - general matrix product                                  *
c     dspev - eigenanalysis of real symmetric matrix in packed storage*
c     dgesvd - singular value decomposition of a real general marix   *
c     dsyev - eigenanalysis of a real symmetric matrix                *         
c     dsyrk - symmetric matrix update                                 *
c                                                                     *
c On SGI machines LAPACK library is specified by f77 compiler option  *
c -lcomplib.sgimath                                                   *
c                                                                     *
c**********************************************************************
c                                                                     *
c     DIMENSIONS:                                                     *
c                                                                     *
c     nr,nc,nfo,z                    - NDIMZ                          *
c     h0                             - NDIM*(NDIM+1)/2                *
c     nq,comment,nblo                - NBL                            *
c     fq,fqc                         - NBLxNDIM                       *
c     fi,ncc                         - NFC                            *
c     x                              - 3xNFC                          *
c     g,w,u,t                        - NFCxNFC                        *
c     eigval                         - NDIM                           *
c                                                                     *
c**********************************************************************

      program dt
      implicit real*8(a-h,o-z)
      include 'Sizes'
      character*20 name,cr
      character*80 comment(nbl)
      character*4 atom(nfc)
      logical ve
      dimension w(nfc,nfc),g(nfc,nfc),u(nfc,nfc),t(nfc,nfc),h0(nkdim),
     . fqc(nbl,ndim),fq(nbl,ndim),ncc(nfc),fi(nfc),nblo(nbl),x(3,nfc),
     . eigval(ndim),nro(4),nco(4),dat(4)
      equivalence (g(1,1),eigval(1))
      common /hkmatrices/ nr(ndimz),nc(ndimz),nfo(ndimz),z(ndimz)
c**********************************************************************
      if(iargc().lt.3) then 
       print *
       print *,' /--------------------------------------------------\\'
       print *,'|                  DDDDDDDD   TTTTTTTTT              |'
       print *,'|                  D       D  T   T   T              |'
       print *,'|                  D       D      T                  |'
       print *,'|                  D       D      T                  |'
       print *,'|                  D       D      T                  |'
       print *,'|                  D       D      T                  |'
       print *,'|                  DDDDDDDD       T                  |'
       print *,'|                                                    |'
       print *,'|                     version 2.0                    |'
       print *,'|                                                    |'
       print *,'|         Generation of Displacement Coordinates     |'
       print *,'|                                                    |'
       print *,'|                                                    |'
       print *,'|                           by Robert Fraczkiewicz   |'
       print *,'|                      University of Houston, 1992-4 |'
       print *,'\\--------------------------------------------------/ '
       print *
       print *,'______________________________________________________'
       print *
       print *,'SYNOPSIS: '
       print *
       print *,'      dt2 [-v] filename nmol factor'
       print *
       print *,'PARAMETERS: '
       print *
       print *,'      filename - core name of the input and output'
       print *,'                 files. Its length cannot exceed 14'
       print *,'                 characters. Appropriate extensions'
       print *,'                 are added by default.'
       print *
       print *,'      nmol - number of molecule (as in ".rein" file)'
       print *
       print *,'      factor - scale factor by which each atomic'
       print *,'               displacement is multiplied'
       print *
       print *,'DESCRIPTION:'
       print *
       print *,'      Program calculates atomic internal and Cartesian'
       print *,'      displacements (commonly known as "eigenvectors")'
       print *,'      from normal mode displacements.'
       print *
       print *,'OPTIONS:'
       print *
       print *,'  -v  verbose flag. Program will produce more output'
       print *,'      to the screen.'
       print *
       print *,'______________________________________________________'
       stop 7
      endif
      ve=.false.
      narg=1
      call getarg(narg,name)
      narg=narg+1
      if(name.eq.'-v') then
       ve=.true.
       call getarg(narg,name)
       narg=narg+1
      endif
      l_name=namlen(name)
      call getarg(narg,cr)
      narg=narg+1
      read (cr,*) nmol
      call getarg(narg,cr)
      narg=narg+1
      read (cr,*) razy
      if(ve) print *,narg-1,' arguments detected'
c**************Read and normalize u-matrix*****************************
      print *,'Reading '//name(1:l_name)//'.umat'
      open(unit=3,file=name(1:l_name)//'.umat',err=2991,status='old') 
      read(3,*) nnt,ns,nsb
      read(3,*)(nblo(i),i=1,nsb)
      do i=1,ns
       do j=1,nnt
        u(i,j)=0.d0
       enddo
      enddo
      do i=1,ns
       read(3,*) nzr,(ncl,u(i,ncl),l=1,nzr)
      enddo
      close(unit=3)
      do i=1,ns
       du=0.0d0
       do j=1,nnt
        du=du+u(i,j)*u(i,j)
       enddo
       du=sqrt(du)
       do j=1,nnt
        u(i,j)=u(i,j)/du
       enddo
      enddo
c**************Read an initial set of force constants*****************
      print *,'Reading '//name(1:l_name)//'.rein'
      open(unit=7,file=name(1:l_name)//'.rein',err=9990,status='old') 
      read(7,*) nf
      read(7,*) (fi(i),i=1,nf)
      read(7,*) nvar
      if(nvar.eq.nf) then
       nvar=nf
       do i=1,nf
        ncc(i)=i
       enddo
      else
       read(7,*) (ncc(i), i=1,nvar)
      endif
      read(7,*) nb
      if(nb.ne.nsb-1) then
       print *,'number of symmetry blocks in '
     .//name(1:l_name)//'.rein is wrong'
       print *,'nb = ',nb
       print *,'nsb-1 = ',nsb-1
       stop 6
      endif
      open(unit=8,file=name(1:l_name)//'.int',status='unknown') 
      ncolu=0
      do i=1,nb
c**************Read experimental frequencies ******************
c nblok - number of symmetry block, nmol - number of molecule *
c**************************************************************
       read(7,200) nblok,nml,comment(i)
  200  format(i2,1x,i2,1x,a80)
       if(nblok.ne.i) then
        print *,'wrong block number in '//name(1:l_name)//'.rein'
        print *,'nblock = ',nblok
        print *,comment(i)
        stop 7
       endif
       if(nml.ne.nmol) then
        print *,'wrong molecule number in '
     .//name(1:l_name)//'.rein'
        print *,'nmol = ',nmol
        print *,comment(i)
        stop 7
       endif
       read(7,*) nfq,nq
       nbbb=nblo(i+1)-nblo(i)
       if(nq.ne.nbbb) then
        print *,'the dimension of the ',i,'-th block = ',nq,' for'
        print *,comment(i)
        print *,'doesn''t match that of corresponding u-matrix block'
        print *,'equal to ',nbbb
        stop 7
       endif
       read(7,*) (fq(i,j), j=1,nfq)
c**************Read and diagonalize a G-matrix block************
       call getname(name,l_name,nblok,nmol,cr,l_cr)
       call readg(cr,l_cr,i,nfq,w,nfc,nq,npom,ve)
c**************Read a Z matrix block**********************************
c*Z matrix must be ordered in the descending order of force constants*
c*********************************************************************
       call getname(name,l_name,nblok,0,cr,l_cr)
       print *,'Reading '//cr(1:l_cr)//'.zmat'
       open(unit=9, file=cr(1:l_cr)//'.zmat', status='old',
     .      form='unformatted')
       read(9) noz
       read(9)(nr(l),nc(l),nfo(l),z(l),l=1,noz)
       close(unit=9)
c*************Prepare H matrix**************************************
       do k=1,nq
        do l=1,nq
         g(k,l)=0.0
        enddo
       enddo
   77  do j=1,noz
        nrow=nr(j)
        ncol=nc(j)
        g(nrow,ncol)=g(nrow,ncol)+z(j)*fi(nfo(j))
        g(ncol,nrow)=g(nrow,ncol)
       enddo
       call dsymm('L','U',nq,npom,1.0d0,g,nfc,w,nfc,0.d0,t,nfc)
       call dgemm('T','N',npom,npom,nq,1.0d0,w,nfc,t,nfc,0.d0,
     .             g,nfc)
       np=0
       do l=1,npom
        do k=1,l
          h0(np+k)=g(k,l)
        enddo
        np=np+l
       enddo
c******Transformation matrix from normal to internal coordinates***
c                          t
c*********************T = U *V1*sqrt(gamma) ***********************
       do k=1,nnt
        do j=1,npom
         r=0.d0
         do l=1,nq
          icol=nblo(i)+l-1
          r=r+u(icol,k)*w(l,j)
         enddo
         t(k,j)=r
        enddo
       enddo
c**************Diagonalize H and find matrix C*********************
       if(ve) print *,'  == MAIN ==  Begin diagonalization'
       call dspev('V','U',npom,h0,eigval,w,nfc,z,info)
       if(ve) then
        print *,'  == DSPEV ==  Error code on return : ',info
        write(*,'(a,f6.0)')
     .         '   == DSPEV ==  Optimal value of lwork : ',z(1)
        print *,'  == DSPEV ==  Current value of lwork : ',ndimz
       endif                
c*****Find and write internal coordinate displacements*************
c
c******************* U = T*C **************************************
       do k=1,nnt
        do j=1,npom
         r=0.d0
         do l=1,npom
          r=r+t(k,l)*w(l,j)
         enddo
         icol=ncolu+j
         u(icol,k)=r
        enddo
       enddo
       print *,'Writing '//name(1:l_name)//'.int'
       write(8,*) 'I n t e r n a l  c o o r d i n a t e s'
       write(8,*) 'displacement vectors for ',i,'-th block'
       write(8,'(1x,a80)') comment(i)
       do j=npom,1,-1
        icol=ncolu+j
        write(8,*)'----------------------------------------------'
        write(8,*) 'Frequency observed   ',fq(i,npom-j+1),' cm-1'
        fqc(i,j)=sqrt(abs(eigval(j))/factor)
        write(8,*) 'Frequency calculated ',fqc(i,j),' cm-1'
        write(8,800) (k,u(icol,k), k=1,nnt)
       enddo
       print '(a,i3)','Done with block ',i
       ncolu=ncolu+npom
       nblo(i)=npom
      enddo
      close(unit=7)
      close(unit=8)
  800 format(5(1x,i3,1x,f11.5))
c**************Read and decompose B-matrix************************
c                    +
c************** T = B  *******************************************
      print *,'Reading '//name(1:l_name)//'.bmat'
      open(unit=10,file=name(1:l_name)//'.bmat',err=2992,status='old',
     .     form='unformatted')
      read(10) nib,n3n
      do i=1,nnt
       do j=1,n3n
        t(i,j)=0.d0
       enddo
      enddo
      read(10) (nr(k),nc(k),z(k),k=1,nib)
      close(unit=10)
      do l=1,nib
       t(nr(l),nc(l))=z(l)
      enddo
      call dgesvd('s','s',nnt,n3n,t,nfc,fi,g,nfc,w,nfc,z,ndimz,info)
      if(ve) then
       print *,' == DGESVD == B-matrix SVD. Error = ',info
       print *,' == DGESVD == Optimal lwork = ',z(1)
       print *,' == DGESVD == Actual lwork  = ',ndimz
      endif
      nonzero=0
      do i=1,min0(nnt,n3n)
       if(abs(fi(i)).lt.eps) then
        goto 11
       else
        nonzero=nonzero+1
       endif
      enddo
   11 do i=1,n3n
       do j=1,nnt
        p=0.d0
        do l=1,nonzero
         p=p+w(l,i)*g(j,l)/fi(l)
        enddo
        t(i,j)=p
       enddo
      enddo
c**************Read and decompose beta matrix*************************
c                             +
c*************** W = (I3n-beta *beta)*T ******************************
      call getname(name,l_name,nmol,0,cr,l_cr)
      print *,'Reading '//cr(1:l_cr)//'.beta'
      open(unit=10,file=cr(1:l_cr)//'.beta',err=2993,status='old')
      do i=1,6
       do j=1,nnt
        g(i,j)=0.d0
       enddo
      enddo
    8 read(10,4)(nro(l),nco(l),dat(l),l=1,4)
    4 format(4(2i3,g15.8))
      do l=1,4
       if(nro(l).eq.-1) goto 273
       g(nro(l),nco(l))=dat(l)
      enddo
      goto 8
  273 close(unit=10)
      call dgesvd('n','s',6,n3n,g,nfc,fi,u,nfc,w,nfc,z,ndimz,info)
      if(ve) then
       print *,' == DGESVD == Beta matrix SVD. error = ',info
       print *,' == DGESVD == Optimal lwork = ',z(1)
       print *,' == DGESVD == Actual lwork  = ',ndimz
      endif
      do i=1,n3n
       g(i,i)=1.d0
       do j=i+1,n3n
        g(i,j)=0.d0
       enddo
      enddo
      call dsyrk('U','T',n3n,6,-1.d0,w,nfc,1.d0,g,nfc)
      call dsymm('L','U',n3n,nnt,1.d0,g,nfc,t,nfc,0.d0,w,nfc)
c***********Normal-to-Cartesian transformation***************
c                     t
c**************T = W*U **************************************
      call dgemm('N','T',n3n,ncolu,nnt,1.d0,w,nfc,u,nfc,0.d0,t,nfc)
c**************Read X-matrix***************************************
      print *,'Reading '//name(1:l_name)//'.xmat'
      open(unit=4, file=name(1:l_name)//'.xmat', err=2994, status='old') 
      i=0
  104 i=i+1
      read(4,939,err=112) atom(i),(x(j,i), j=1,3)
      goto 104
  939 format(1x,a4,3(2x,f14.10))
  112 close(unit=4)
      noat=n3n/3
      if(i-1.ne.noat) then
       print *,'*--*--*-- error --*--*--*'
       print *,'the number of an atom in '
     .//name(1:l_name)//'.xmat file = ',i-1
       print *,'number of cartesian coordinates = ',n3n
       stop 8
      endif
      do i=1,noat
       cr=atom(i)
       ic=ichar(cr(2:2))
       if(ic.ge.65.and.ic.le.122) then
        atom(i)=cr(1:2)
       else
        atom(i)=cr(1:1)
       endif
      enddo
c**************Write Cartesian displacements**************************
      print *,'Writing '//name(1:l_name)//'.xyz'
      open(unit=8, file=name(1:l_name)//'.xyz', status='unknown') 
      ncolt=0
      ncount=0
      do i=1,nb
       do j=nblo(i),1,-1
        ncount=ncount+1
        write(8,*) noat
        write(8,901) ncount,comment(i),fq(i,nblo(i)-j+1),fqc(i,j)
        ncoj=ncolt+j
        do k=1,noat
         k31=3*(k-1)
         write(8,900) atom(k),(x(l,k), l=1,3),
     .               (razy*t(k31+l,ncoj),l=1,3)
        enddo
       enddo
       ncolt=ncolt+nblo(i)
      enddo
  900 format(3x,a4,6(1x,f9.5))
  901 format(1x,i3,1x,a30,1x,'obs. ',f7.2,' cm-1, calc. ',f7.2,' cm-1')
      close(unit=8)
      stop 1
c  ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 2991 print *
      print *,'Input operation unsuccesful. Can"t find u-matrix file'
      stop 2
 2992 print *
      print *,'Input operation unsuccesful. Can"t find b-matrix file'
      stop 3
 2993 print *
      print *,'Input operation unsuccesful. Can"t find beta file'
      stop 4
 2994 print *
      print *,'Input operation unsuccesful. Can"t find x-matrix file'
      stop 5
 9990 print *
      print *,'Input operation unsuccesful. Check the name and format'
      print *,'of the data file. Don"t input extension of filename !'
      stop 6
c  ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      end

      subroutine readg(cr,l_cr,i,nfq,w,ldw,ncoln,npom,ve)
      implicit real*8(a-h,o-z)
      include 'Sizes'
      character*20 cr
      logical ve
      dimension gg(nint,nint),eigval(nint),w(ldw,ncoln),work(lwork)
      common /hkmatrices/ nr(ndimz),nc(ndimz),nfo(ndimz),z(ndimz)
c***************Read G-matrix block******************************
       print *,'Reading '//cr(1:l_cr)//'.gmat'
       open(unit=17,file=cr(1:l_cr)//'.gmat',status='old',form
     . ='unformatted')
       read(17) nel
       read(17) (nr(l),nc(l),z(l),l=1,nel)   
       close(unit=17)
c***************Initialize G-matrix******************************
       do k=1,ncoln
        do l=1,ncoln
         gg(k,l)=0.d0
        enddo
       enddo
       do l=1,nel
        gg(nr(l),nc(l))=z(l)
        gg(nc(l),nr(l))=z(l)
       enddo
c****************Diagonalize G matrix***********************
       call dsyev('V','U',ncoln,gg,nint,eigval,work,lwork,info)
       if(ve) then
        print *,'  == READG == '
        print *,'  == DSYEV ==  Error code on return : ',info
        write(*,'(a,f6.0)')
     .          '   == DSYEV ==  Optimal value of lwork : ',work(1)
        print *,'  == DSYEV ==  Current value of lwork : ',lwork
       endif
c****************Calculate scaled eigenvectors***************
       nzero=0
       do k=1,ncoln
        kk=ncoln-k+1
        if(dabs(eigval(kk)).lt.5.0d-5) then
         nzero=nzero+1
         eigval(kk)=0.d0
        endif
        do j=1,ncoln
         w(j,k)=gg(j,kk)*dsqrt(eigval(kk))
        enddo
       enddo
       npom=ncoln-nzero
       if(npom.ne.nfq) then
        print *,'Number of frequencies for',i,' block does'
        print *,'not match the rank of G-matrix = ',npom
        stop 3
       endif
      end
