C**********************************************************************
C                 Copyright 1992-1996, Robert Fraczkiewicz            *
C                           All rights reserved.                      *
C**********************************************************************
C This program is free software; you can redistribute it and/or modify*
C it under the terms of the GNU General Public License as published by*
C the Free Software Foundation; either version 1, or (at your option) *
C any later version.                                                  *
C                                                                     *
C This program is distributed in the hope that it will be useful,     *
C but WITHOUT ANY WARRANTY; without even the implied warranty of      *
C MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU    *
C General Public License for more details.                            *
C                                                                     *
C You should  have received a copy of the GNU General Public License  *
C along with this program; if not, write to the Free Software         *
C Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.           *
c**********************************************************************
c                                                                     *
c    Program "Secular Equation Solver" by Robert Fraczkiewicz         *
c                      Dec 27, 94, version 6.1                        *
c                                                                     *
c**********************************************************************
c Program FR, working in the interactive mode, solves secular equation*
c providing with vibrational frequencies, isotopic shifts and PED's.  *
c It also calculates significance matrices and PED and isotopic shift *
c sensitivity. Any set of free force constants can be changed         *
c interactively and followed by a display of its impact on any of the *
c quantities mentioned above. Any block-molecule combination can be   *
c specified in the input file. WARNING: For isotopic shift sensitivity*
c calculations one must use the same symmetry block for each molecule.*
c**********************************************************************
c                                                                     *
c     INPUT FILES:                                                    *
c                                                                     *
c     ".rein" - main input file containing force field and obseved    *
c               vibrational frequencies                               *
c     ".zmat" - Z-matrix (output of ZSYM4)                            *
c     ".gmat" - G-matrix (output of GMAT4)                            *
c     ".force" - description of individual force constant parameters  *
c                                                                     *
c**********************************************************************
c                                                                     *
c     OUTPUT FILES:                                                   *
c                                                                     *
c     ".frout" - main output file. Contains old and new force field,  *
c                observed and calculated frequencies and PED's        *
c     ".fi"    - new force field                                      *
c     ".ped"   - binary file (usually large) containing full PED      *
c                vectors. Input for PEDSORT.                          *
c                                                                     *
c**********************************************************************
c                                                                     *
c     BLAS AND LAPACK ROUTINES:                                       *
c                                                                     *
c     dsymm - symmetric matrix product                                *
c     dgemm - general matrix product                                  *
c     dgemv - matrix-vector product                                   *
c     dspmv - (symmetric) packed matrix-vector product                *
c     dspev - eigenanalysis of real symmetric matrix in packed storage*
c     dspevx - eigenanalysis of real symmetric matrix in packed       *
c              storage (an expert routine)                            *
c     dsyev - eigenanalysis of a real symmetric matrix                *         
c     ddot - vector dot product                                       *
c                                                                     *
c On SGI machines LAPACK library is specified by f77 compiler option  *
c -lcomplib.sgimath                                                   *
c                                                                     *
c**********************************************************************
c                                                                     *
c     DIMENSIONS:                                                     *
c                                                                     *
c     nr,nc,z,nfo                    - NBLxNDIMZ                      *
c     q                              - NDIMxNDIMxNBL                  *
c     h0                             - [NDIM*(NDIM+1)/2]xNBL          *
c     nq,nz,comment,nrw              - NBL                            *
c     fq                             - NBLxNDIM                       *
c     eig                            - NDIMxNBL                       *
c     fi,fi0,ncc,index,nchanged      - NFC                            *
c     rr                             - NINTxNFC                       *
c     g,t                            - NFCxNFC                        *
c     w                              - NFCxNFCxNBL                    *
c     vect                           - NINT                           *
c     numb                           - NBLxNFC                        *
c                                                                     *
c**********************************************************************

      program fr
      implicit real*8(a-h,o-z)
      include 'Sizes'
      character*20 name,cr,crz
      character*80 comment(nbl),lines(2)
      logical change,tmpin,ve,fcs
      integer typ
      dimension g(nfc,nfc),t(nfc,nfc),nchanged(nfc)
      real tarray(2)
      common /hkmatrices/ nr(nbl,ndimz),nc(nbl,ndimz),numb(nbl,nfc),
     .                    z(nbl,ndimz)
     .       /qmat/ nq(nbl),nz(nbl),nfo(nbl,ndimz),q(ndim,ndim,nbl)
     .       /hmat/ h0(nkdim,nbl),fi(nfc),fi0(nfc)
     .       /pedm/ nzz,noz,rr(nint,nfc),eig(ndim,nbl),fq(nbl,ndim)
     .       /sigm/ ncc(nfc),index(nfc)
     .       /position/ w(nfc,nfc,nbl),vect(nint),nrw(nbl)
      data lines /'Potential energy distribution WRT force constants:',
     .       'Potential energy distribution WRT symmetry coordinates:'/

c**********************************************************************
      ve=.false.
      if(iargc().eq.0) then
       print *,'^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^'
       print *
       print *,'                    FFFFF   RRRRR'
       print *,'                    F       R   R'
       print *,'                    FFF     RRRRR'
       print *,'                    F       R R'
       print *,'                    F       R  R'
       print *
       print *,'                     Version 6.1'
       print *
       print *,'      Solution of the Direct Vibrational Problem'
       print *
       print *,'______________________________________________________'
       print *
       print *,'SYNOPSIS:'
       print *
       print *,'     fr [-v] filename'
       print *
       print *,'PARAMETERS: '
       print *
       print *,'      filename - core name of the input and output'
       print *,'                 files. Its length cannot exceed 14'
       print *,'                 characters. Appropriate extensions'
       print *,'                 are added by default.'
       print *
       print *,'DESCRIPTION:'
       print *
       print *,'      Program FR, working in the interactive mode,'
       print *,'      solves secular equation providing with '
       print *,'      vibrational frequencies, isotopic shifts and'
       print *,'      PED''s. It also calculates significance matrices'
       print *,'      and isotopic shift sensitivity. Any set of free'
       print *,'      force constants can be changed interactively and'
       print *,'      followed by a display of its impact on any of the'
       print *,'      quantities mentioned above. Any block-molecule '
       print *,'      combination can be specified in the input file.'
       print *,'      WARNING: For isotopic shift sensitivity calcu-'
       print *,'      lations one must use the same symmetry block for'
       print *,'      each molecule !'
       print *
       print *,'OPTIONS:'
       print *
       print *,'  -v  verbose flag. Program will produce more output'
       print *,'      to the screen'
       print *
       print *,'______________________________________________________'
       stop 1
      endif
      call getarg(1,name)
      if(name.eq.'-v') then
       ve=.true.
       call getarg(2,name)
      endif
      l_name=namlen(name)
c**************Read an initial set of force constants*****************
      open(unit=9,file=name(1:l_name)//'.rein',err=9990,status='old') 
      read(9,*) nf
      read(9,*) (fi(i),i=1,nf)
      read(9,*) nvar
      if(nvar.lt.0) then
       nvar=nf+nvar
       nfixed=nf-nvar
       read(9,*) (nr(1,i),i=1,nfixed)
       l=1
       j=1
       i=0
       do while(l.le.nfixed.or.j.le.nvar)
        i=i+1
        if(i.eq.nr(1,l)) then
         l=l+1
        else
         ncc(j)=i
         j=j+1
        endif
       enddo
      else
       nfixed=nf-nvar
       if(nfixed.eq.0) then
        do i=1,nvar
         ncc(i)=i
        enddo
       else
        read(9,*) (ncc(i), i=1,nvar)
       endif
      endif 
      read(9,*) nb
      f=0
      do i=1,nb
c**************Read experimental frequencies*******************
c nblok - number of symmetry block, nmol - number of molecule *
c**************************************************************
       read(9,'(i2,1x,i2,1x,a80)') nblok,nmol,comment(i)
       read(9,*) nfq,nq(i)
       nrw(i)=nq(i)
       read(9,*) (fq(i,j), j=1,nfq)
c**************Read and diagonalize a G-matrix block************
       call getname(name,l_name,nblok,nmol,cr,l_cr)
       call readg(cr,l_cr,i,nfq,w(1,1,i),nfc,nq(i),npom,ve)
c**************Read a Z matrix block**********************************
c*Z matrix must be ordered in the descending order of force constants*
c*********************************************************************
       call getname(name,l_name,nblok,0,cr,l_cr)
       print *,'Reading '//cr(1:l_cr)//'.zmat'
       open(unit=7, file=cr(1:l_cr)//'.zmat', status='old',form=
     . 'unformatted')
       read(7) noz
       read(7)(nr(i,l),nc(i,l),nfo(i,l),z(i,l),l=1,noz)
       close(unit=7)
c*********************Order matrices Zk*******************
   30  do k=1,nq(i)
        do l=1,nq(i)
         g(k,l)=0.0
        enddo
       enddo
   70  change=.false.
       do l=1,noz-1
        ll=l+1
        tmpin=in(nfo(i,l),ncc,nvar,nfc)
        if(.not.tmpin) then
         tmpin=in(nfo(i,ll),ncc,nvar,nfc)
         if(tmpin) then
          change=.true.
          n1=nr(i,l)
          n2=nc(i,l)
          n3=nfo(i,l)
          zz=z(i,l)
          nr(i,l)=nr(i,ll)
          nc(i,l)=nc(i,ll)
          nfo(i,l)=nfo(i,ll)
          z(i,l)=z(i,ll)
          nr(i,ll)=n1
          nc(i,ll)=n2
          nfo(i,ll)=n3
          z(i,ll)=zz
         endif
	endif
       enddo
       if(change) goto 70 
       k=0
   80  k=k+1        
       if(k.le.noz) then
        tmpin=in(nfo(i,k),ncc,nvar,nfc)
        if(tmpin) goto 80
       endif
       nzz=k-1
c******************Calculate H0 matrix***********************
       do j=1,noz
        nrow=nr(i,j)
        ncol=nc(i,j)
        g(nrow,ncol)=g(nrow,ncol)+z(i,j)*fi(nfo(i,j))
        g(ncol,nrow)=g(nrow,ncol)
       enddo
       call dsymm('L','U',nq(i),npom,1.0d0,g,nfc,w(1,1,i),nfc,
     .             0.d0,t,nfc)
       call dgemm('T','N',npom,npom,nq(i),1.0d0,w(1,1,i),nfc,
     .             t,nfc,0.d0,g,nfc)
       np=0
       do l=1,npom
        do k=1,l
          h0(np+k,i)=g(k,l)
        enddo
        np=np+l
       enddo
c**************Order active force constants********************
       do k=1,nvar
        numb(i,k)=0.d0
       enddo
       k=nfo(i,1)
       nz(i)=0
       do l=1,nzz
        if(nfo(i,l).ne.k) then
         nz(i)=nz(i)+1
         nfo(i,nz(i))=nfo(i,l-1)
         numb(i,nz(i))=l-1
         k=nfo(i,l)
        endif
       enddo
       if(nzz.gt.0) then 
        nz(i)=nz(i)+1
        nfo(i,nz(i))=nfo(i,nzz)
        numb(i,nz(i))=nzz
       endif
       nq(i)=npom
       write(*,'(a,i3)') 'Done with block ',i
      enddo
      close(unit=9)
c**************Save old force constants**********************
      do i=1,nf
       fi0(i)=fi(i)
      enddo        
c**************Diagonalize H and construct q vectors***********
      do i=1,nb
       write(*,'(a,i3,a)') 'Diagonalizing matrix H(',i,')'
       call diagh('V',i,0,eig(1,i),ve)
      enddo
c*******************Timing*******************************
      if(ve) then
       eti=etime(tarray)
       t1=tarray(1)
       t2=tarray(2)
       tt=t1+t2
       print *
       print *,'  == ETIME ==    User time: ',t1,' seconds'
       print *,'               System time: ',t2,' seconds'
       print *,'                Total time: ',tt,' seconds'
       print *
       rewind 18
       rewind 19
      endif
c********************Print frequencies**********************
      nn=0
      typ=-1
      fcs=.false.
  100 do i=1,nf
       fi0(i)=fi(i)
      enddo        
      fff=0.d0
      do i=1,nb
       ffff=0.d0
       write(*,96) 
       print *,comment(i)
       do j=1,nq(i),8
        jj=j+7
        if(jj.gt.nq(i)) jj=j+mod(nq(i),8)-1
        do kk=j,jj
         if(fq(i,kk).gt.0.d0) then
          r=factor*fq(i,kk)*fq(i,kk)-eig(nq(i)-kk+1,i)
          ffff=ffff+r*r
         endif
        enddo
        write(*,90) (fq(i,k), k=j,jj)
        write(*,91) (sqrt(abs(eig(nq(i)-k+1,i))/factor), k=j,jj)
       enddo
       write(*,96)
       print *,'block ',i,', square deviation = ',sqrt(ffff)
       fff=fff+ffff
      enddo
      write(*,96)
      print *,'total square deviation = ',sqrt(fff)
      write(*,96)
   90 format(1x,'obsd :',8(1x,f8.2))
   91 format(1x,'calc :',8(1x,f8.2))
   96 format
     .('------------------------------------------------------------')
c************Display main menu*****************************
  101 print *,'Main menu'
      print *,' positive number -> force constant to be changed'
      print *,' 0               -> recalculate frequencies'
      print *,'-1               -> write freq-s., PEDs and quit'
      print *,'-2               -> calculate significance matrix'
      print *,'-3               -> write freq-s. only and quit'
      print *,'-4               -> calculate PEDs'
      print *,'-5               -> calculate shift sensitivity'
      print *,'-6               -> unconditional exit'
      read(*,*) kk
      if(kk) 104,103,102
c************Obtain new set of force constants**************
  102 tmpin=in(kk,ncc,nvar,nfc)
      if(.not.tmpin) then
       write(*,92) kk,fi(kk)
       print *, kk,' - th force constant is fixed !'
       go to 101
      endif
      write(*,92) kk,fi(kk)
   92 format(1x,'Old value of fi(',i3,') = ',f7.4)
      print *,'Enter new value :'
      read(*,*) fi(kk)
      tmpin=in(kk,nchanged,nn,nfc)
      if(.not.tmpin) then
       nn=nn+1
       nchanged(nn)=kk
      endif
      go to 101
c**************Form new H matrix and diagonalize it*********
  103 do i=1,nb
       write(*,'(a,i3,a)') 'Diagonalizing matrix H(',i,')'
       call diagh('V',i,0,eig(1,i),ve)
      enddo
      go to 100
c***************Option selector*******************************
  104 go to (999,107,999,300,400,998) iabs(kk)
      go to 100
c************Calculate significance matrix********************
  107 print *,'Significance matrix menu (level 1)'
      print *,' positive number -> number of symmetry block;
     . max=',nb
      print *,' 0               -> go back to main menu'
      print *,'-1               -> calc. force consts. sign. mtx.'
      print *,'-2               -> calc. normal mode  sign. mtx.(def)'
      read(*,*) kk
      if(kk.eq.0) go to 100
      if(kk.gt.0) go to 108
      go to (121,122) iabs(kk)
      go to 107
  121 fcs=.true.
      print *
      print *,'Force constants significance matrix will be calculated'
      print *
      go to 107
  122 fcs=.false.
      print *
      print *,'Normal modes significance matrix will be calculated'
      print *
      go to 107
  108 if(kk.gt.nb) go to 107
      nblock=kk
      print *,'Enter force constant increment : (0 -> go back)'
      read(*,*) dump
      if(dump.eq.0.d0) go to 107
      call sig(dump,nblock,nvar,ve,fcs)
  109 print *,'Significance matrix menu (level 2)'
      if(fcs) then      
       print *,' positive number -> number of force constant'
       print '(a,i3)','                    to be examined; max=',nf
      else
       print *,' positive number -> number of mode to be examined;
     . max=',nq(nblock)
      endif
      print *,' 0               -> go back to previous menu'
      print *,'-1               -> calc. force constants. sign. mtx.'
      print *,'-2               -> calc. normal mode  sign. mtx.(def)'
      print *,'-3               -> write significance mtx. to a file'
      read(*,*) kk
      if(kk.eq.0) go to 107
      if(kk.gt.0) go to 110
      go to (123,124,130) iabs(kk)
      go to 109
  123 fcs=.true.
      print *,'Force constants significance matrix will be calculated'
      go to 109
  124 fcs=.false.
      print *,'Normal modes significance matrix will be calculated'
      go to 109
  110 if(fcs) then
       if(kk.gt.nf) go to 109
      else
       if(kk.gt.nq(nblock)) go to 109
      endif
      tmpin=in(kk,ncc,nvar,nfc)
      if(.not.tmpin) then
       write(*,92) kk,fi(kk)
       print *, kk,' - th force constant is fixed !'
       go to 109
      endif
      write(*,96)
      write(*,'(1x,a)') comment(nblock)
      write(*,96) 
      call order(fcs,kk,ncut,nblock,nvar,ve)
      if(fcs) then
       write(*,'(1x,a,i3,a,f7.4)') 'Force constant fi(',kk,
     .  ') = ',fi(kk)
       do k=1,ncut
        write(*,'(1x,a,i3,a,f8.2,a,f8.2,a,f9.4)')
     .  'Mode ',index(k),
     .  ' Observed ',fq(nblock,index(k)),
     .  ' Calculated ',
     .   sqrt(abs(eig(nq(nblock)-index(k)+1,nblock))/factor),
     .  ' Significance ',rr(index(k),kk)
       enddo
       write(*,96)
       go to 109
      else
       write(*,196) kk,fq(nblock,kk)
       write(*,197) sqrt(abs(eig(nq(nblock)-kk+1,nblock))/factor)
       write(*,96)
       write(*,99) (index(m),
     .             rr(kk,index(m)), m=1,ncut)
       write(*,96)
       go to 109
      endif
  130 print *,'== File name :'
      read(*,'(a20)') crz
      l_crz=namlen(crz)
      print *,'Writing '//crz(1:l_crz)
      open(unit=11, file=crz(1:l_crz), status='unknown')
      write(11,96)
      write(11,'(1x,a)') comment(nblock)
      write(11,96)
      if(fcs) then
       do m=1,nvar
        kk=ncc(m)
        call order(fcs,kk,ncut,nblock,nvar,ve)
        write(11,'(1x,a,i3,a,f7.4)') 'Force constant fi(',kk,
     .   ') = ',fi(kk)
        do k=1,ncut
         write(11,'(1x,a,i3,a,f8.2,a,f8.2,a,f9.4)')
     .   'Mode ',index(k),
     .   ' Observed ',fq(nblock,index(k)),
     .   ' Calculated ',
     .   sqrt(abs(eig(nq(nblock)-index(k)+1,nblock))/factor),
     .   ' Significance ',rr(index(k),kk)
        enddo
        write(11,96)
       enddo 
      else
       do kk=1,nq(nblock)
        call order(fcs,kk,ncut,nblock,nvar,ve)
        write(11,196) kk,fq(nblock,kk)
        write(11,197) sqrt(abs(eig(nq(nblock)-kk+1,nblock))/factor)
        write(11,96)
        write(11,99) (index(m),
     .              rr(kk,index(m)), m=1,ncut)
        write(11,96)
       enddo
      endif
      close(unit=11)
      go to 109
   99 format(5(1x,i3,1x,f9.4))
c**************Calculate potential energy distribution***************
  300 print *,'Potential energy distribution menu'
      if(nb.eq.1) then
       nblock=1
       print *,' Positive number -> number of mode to be examined;
     . max=',nq(nblock)
      else
       print *,' Positive number -> block number; max=',nb
      endif
      print *,' 0               -> go back to main menu'
      print *,'-1               -> force constant PEDs (default)'
      print *,'-2               -> symmetry coordinate PEDs'
      print *,'-3               -> F-matrix PED sensitivity'
      read(*,*) nblock
      if(nblock) 301,100,302
  301 iout2=0
      typ=nblock
      print *
      print '(a,i3,a)','Option ',typ,' was chosen.'
      print *
      goto (300,300,305) iabs(nblock)
  305 print *,'PED sensitivity menu (level 2)'
      print *,'-1               -> append results to a file'
      print *,'-2               -> screen output only (def)'
      read(*,*) nsen
      goto (306,307) iabs(nsen)
      goto 305      
  306 iout2=16
      close(unit=16)
      print *,'== File name :'
      read(*,'(a20)') crz
      l_crz=namlen(crz)
      inquire(file=crz(1:l_crz), exist=change, opened=tmpin)
      if(change) then
       open(unit=16, file=crz(1:l_crz), access='append', status='old')
      else
       open(unit=16, file=crz(1:l_crz), access='append', status='new')
      endif
  307 print *,'Enter force constant increment'
      read(*,*) dump
      goto 300      
  302 if(nb.eq.1) then
       nfqcy=nblock
       nblock=1
      else
       if(nblock.gt.nb) goto 300
       print *,'Enter mode number : (0 -> go back); max=',nq(nblock)
       read(*,*) nfqcy
      endif
  303 if(nfqcy.le.0) goto 300
      if(nfqcy.gt.nq(nblock)) goto 304
      write(*,'(a80)') lines(iabs(typ))
      write(*,96)
      write(*,'(1x,a)') comment(nblock)
      write(*,96)
      write(*,196) nfqcy,fq(nblock,nfqcy)
      write(*,197) sqrt(abs(eig(nq(nblock)-nfqcy+1,nblock))/factor)
      write(*,96)
      if(iout2.gt.0) then
       write(iout2,'(a80)') lines(iabs(typ))
       write(iout2,96)
       write(iout2,'(1x,a)') comment(nblock)
       write(iout2,96)
       write(iout2,196) nfqcy,fq(nblock,nfqcy)
       write(iout2,197) 
     .       sqrt(abs(eig(nq(nblock)-nfqcy+1,nblock))/factor)
       write(iout2,96)
      endif
      call ped(6,iout2,nblock,nfqcy,nvar,typ,dump,ve,name,l_name)
      write(*,96)
      if(iout2.gt.0) write(iout2,96)
  304 print *,'Enter mode number : (0 -> go back); max=',nq(nblock)
      read(*,*) nfqcy
      goto 303
c***************Calculate isotopic shift significance*****************
  400 print *,'Enter force constant increment : (0 -> go back)'
      read(*,*) dump
      if(dump.eq.0.d0) go to 107
      iout2=0
  401 print *,' Positive number -> mode number; max=',nq(1)
      print *,' 0               -> go back to main menu'
      print *,'-1               -> append results to a file'
      read(*,*) nfqcy
      if(nfqcy.eq.0) go to 100
      if(nfqcy.lt.-1.or.nfqcy.gt.nq(1)) go to 401
      if(nfqcy.gt.0) goto 403
      iout2=17
      close(unit=17)
      print *,'== File name :'
      read(*,'(a20)') crz
      l_crz=namlen(crz)
      inquire(file=crz(1:l_crz), exist=change, opened=tmpin)
      if(change) then
       open(unit=17, file=crz(1:l_crz), access='append', status='old')
      else
       open(unit=17, file=crz(1:l_crz), access='append', status='new')
      endif
      goto 401
  403 write(*,96)
      write(*,'(1x,a)') comment(1)
      write(*,96)
      write(*,196) nfqcy,fq(1,nfqcy)
      write(*,197) sqrt(abs(eig(nq(1)-nfqcy+1,1))/factor)
      write(*,96)
      if(iout2.gt.0) then
       write(iout2,96)
       write(iout2,'(1x,a)') comment(1)
       write(iout2,96)
       write(iout2,196) nfqcy,fq(1,nfqcy)
       write(iout2,197) 
     .       sqrt(abs(eig(nq(1)-nfqcy+1,1))/factor)
       write(iout2,96)
      endif
      io1=6
      call iso_shifts(io1,iout2,nb,nfqcy,nvar,dump,ve)
      goto 401
c**********************Finish job*****************************
  999 if(typ.eq.-3) typ=-1
      open(unit=8, file=name(1:l_name)//'.frout', status='unknown') 
      open(unit=7, file=name(1:l_name)//'.fi', status='unknown') 
      if(kk.eq.-1) open(unit=10,file=name(1:l_name)//'.ped',
     .                  status='unknown',form='unformatted')
      write(8,*)'Changed force constants : '
      write(8,194)(nchanged(j), j=1,nn)
      write(8,*)'New force constants : '
      write(8,193)(fi(k), k=1,nf)
      write(7,193)(fi(k), k=1,nf)
  193 format(10(1x,f7.4))
  194 format(20(1x,i3))
  195 format(1x,f8.2,4x,f8.2,5x,f8.2)
      write(8,*)'New frequencies : '
      do i=1,nb
       write(8,96) 
       write(8,'(1x,a80)') comment(i)
       write(8,96)
       write(8,*) ' observed   calculated   difference (all in cm-1)'
       do k=1,nq(i)
        km=nq(i)-k+1
        fy=sqrt(abs(eig(km,i))/factor)
        write(8,195) fq(i,k),fy,fq(i,k)-fy
        if(eig(km,i).lt.0.d0) 
     .   print *,'negative eigenvalue !!!!! block ',i,' frq ',k
       enddo
       write(8,96)
       if(kk.eq.-1) then
        write(8,'(a80)') lines(iabs(typ))
        write(8,96)
        write(8,'(1x,a)') comment(i)
        write(8,96)
        goto (250,251,251) iabs(typ)
  250   nvect=nz(i)
        goto 252
  251   nvect=nrw(i)*(nrw(i)+1)/2
  252   write(10) nvect,nq(i),comment(i)
        do k=1,nq(i)
         km=nq(i)-k+1
         write(*,'(a,i3,a,i3,a)')
     .          'Calculating PED of ',k,
     .          '-th mode of the ',i,
     .          '-th block'
         write(8,196) k,fq(i,k)
         write(8,197) sqrt(abs(eig(km,i))/factor)
         call ped(8,0,i,k,nvar,typ,0,.false.,name,l_name)
         write(8,96)
        enddo
       endif
      enddo
  196 format(1x,'Mode ',i3,21x,'Obsd. :',f8.2)
  197 format(30x,'Calc. :',f8.2)
      if(kk.eq.-1) close(unit=10)
      close(unit=7)
      close(unit=8)
c*******************Timing*******************************
  998 if(ve) then
       eti=etime(tarray)
       t1=tarray(1)
       t2=tarray(2)
       tt=t1+t2
       print *
       print *,'  == ETIME ==    User time: ',t1,' seconds'
       print *,'               System time: ',t2,' seconds'
       print *,'                Total time: ',tt,' seconds'
       print *
      endif
      print *,'Finished ... '
      stop 0
c  ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 9990 print *
      print *,'Input operation unsuccesful. Check the name and format'
      print *,'of the data file. Don"t input extension of filename !'
      stop 2
c  ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      end

      subroutine readg(cr,l_cr,i,nfq,w,ldg,ncoln,npom,ve)
      implicit real*8(a-h,o-z)
      include 'Sizes'
      character*20 cr
      logical ve
      dimension gg(nfc,nfc),eigval(nfc),w(ldg,ncoln),work(lwork)
      common /hkmatrices/ nr(nbl,ndimz),nc(nbl,ndimz),numb(nbl,nfc),
     .                    z(nbl,ndimz)
c***************Read G-matrix block******************************
       print *,'Reading '//cr(1:l_cr)//'.gmat'
       open(unit=7,file=cr(1:l_cr)//'.gmat',status='old',form
     . ='unformatted')
       read(7) nel
       read(7) (nr(i,l),nc(i,l),z(i,l),l=1,nel)   
       close(unit=7)
c***************Initialize G-matrix******************************
       do k=1,ncoln
        do l=1,ncoln
         gg(k,l)=0.d0
        enddo
       enddo
       do l=1,nel
        gg(nr(i,l),nc(i,l))=z(i,l)
        gg(nc(i,l),nr(i,l))=z(i,l)
       enddo
c****************Diagonalize G matrix***********************
       call dsyev('V','U',ncoln,gg,ldg,eigval,work,lwork,info)
       if(ve) then
        print *,'  == READG == '
        print *,'  == DSYEV ==  Error code on return : ',info
        write(*,'(a,f6.0)')
     .          '   == DSYEV ==  Optimal value of lwork : ',work(1)
        print *,'  == DSYEV ==  Current value of lwork : ',lwork
       endif
c****************Calculate scaled eigenvectors***************
       nzero=0
       do k=1,ncoln
        kk=ncoln-k+1
        if(dabs(eigval(kk)).lt.5.0d-5) then
         nzero=nzero+1
         eigval(kk)=0.d0
        endif
        do j=1,ncoln
         w(j,k)=gg(j,kk)*dsqrt(eigval(kk))
        enddo
       enddo
       npom=ncoln-nzero
       if(npom.ne.nfq) then
        print *,'Number of frequencies for',i,' block does'
        print *,'not match the rank of G-matrix = ',npom
        stop 3
       endif
      end

      subroutine hkmat(i,k,ldw,ncw,ww,nhk,hk)
      implicit real*8(a-h,o-z)
      include 'Sizes'
c**************Calculate matrices Hk********************
      dimension ww(ldw,ncw),hk(nhk)
      common /hkmatrices/ nr(nbl,ndimz),nc(nbl,ndimz),numb(nbl,nfc),
     .                    z(nbl,ndimz)
      if(k.eq.1) then
       low=0
      else 
       low=numb(i,k-1)
      endif
      npos=0
      do n=1,ncw
       do m=1,n
        r=0.0d0
        do j=low+1,numb(i,k)
         nnr=nr(i,j)
         nnc=nc(i,j)
         r=r+ww(nnr,m)*ww(nnc,n)*z(i,j)
         if(nnr.ne.nnc) r=r+ww(nnc,m)*ww(nnr,n)*z(i,j)
        enddo
        hk(npos+m)=r
       enddo
       npos=npos+n
      enddo
      end
 
      subroutine diagh(jobz,i,nfqcy,eigval,ve)
      implicit real*8(a-h,o-z)
      include 'Sizes'
c*******Diagonalization procedure**************************
      integer nhk
      parameter (nhk=nkdim)
      logical ve
      character*1 jobz
      dimension work(lwork),eigval(ndim),h(nkdim),iwork(lwork),
     .          ifail(1)
      common /qmat/ nq(nbl),nz(nbl),nfo(nbl,ndimz),q(ndim,ndim,nbl)
     .       /hmat/ h0(nkdim,nbl),fi(nfc),fi0(nfc)
     .       /position/ w(nfc,nfc,nbl),vect(nint),nrw(nbl)
      if(ve) print *,'  == DIAGH ==  Preparing matrix H'
      do j=1,nz(i)
       nn=nfo(i,j)
       fc=fi(nn)-fi0(nn)
       if(fc.ne.0.d0) then
        call hkmat(i,j,nfc,nq(i),w(1,1,i),nhk,h)
        np=0
        do jj=1,nq(i)
         do ii=1,jj
           h0(np+ii,i)=h0(np+ii,i)+fc*h(np+ii)
         enddo
         np=np+jj
        enddo
       endif
      enddo
      np=0
      do jj=1,nq(i)
       do ii=1,jj
         h(np+ii)=h0(np+ii,i)
       enddo
       np=np+jj
      enddo
      if(ve) print *,'  == DIAGH ==  Begin diagonalization'
      if(nfqcy.gt.0.and.nfqcy.le.nq(i)) then
       il=nq(i)-nfqcy+1
       call dspevx(jobz,'I','U',nq(i),h,0,0,il,il,0,m,eigval,
     .             q(1,1,i),ndim,work,iwork,ifail,info)
       if(ve) then
        print *,'  == DSPEVX ==  Error code on return : ',info
        print *,'  == DSPEVX ==  Eigenvalues found : ',m
        if(info.ne.0) print *,
     .'  == DSPEVX ==  !! Eigenvector failed to converge: ', ifail(1)
       endif
      else
       call dspev(jobz,'U',nq(i),h,eigval,q(1,1,i),ndim,work,info)
       if(ve) print *,'  == DSPEV ==  Error code on return : ',info
      endif
      end

      subroutine ped(iout,iout2,nblock,nfqcy,nvar,typ,dump,ve,name,
     .               l_name)
      implicit real*8(a-h,o-z)
      include 'Sizes'
c**********Calculation of PED*******************************
      integer nhk,typ
      real*8 cutoff,cutsen
      parameter (nhk=nkdim)
      parameter (cutoff=0.05,cutsen=0.01)
      logical change,ve
      character*20 name
      character*35 fname(nfc)
      dimension h(nkdim),s(nint),nrow(nfc)
      common /qmat/ nq(nbl),nz(nbl),nfo(nbl,ndimz),q(ndim,ndim,nbl)
     .       /hmat/ h0(nkdim,nbl),fi(nfc),fi0(nfc)
     .       /pedm/ nzz,noz,rr(nint,nfc),eig(ndim,nbl),fq(nbl,ndim)
     .       /position/ w(nfc,nfc,nbl),vect(nint),nrw(nbl)
     .       /hkmatrices/ nr(nbl,ndimz),nc(nbl,ndimz),numb(nbl,nfc),
     .                    z(nbl,ndimz)
     .       /sigm/ ncc(nfc),index(nfc)
      nqn1=nq(nblock)-nfqcy+1
      go to (10,20,20) iabs(typ)
c*****************FF parameters PED***************************
   10 do j=1,nz(nblock)
       fc=fi(nfo(nblock,j))
       if(fc.ne.0.d0) then 
        call hkmat(nblock,j,nfc,nq(nblock),w(1,1,nblock),nhk,h)
        call dspmv('U',nq(nblock),1.d0,h,q(1,nqn1,nblock),
     .              1,0.d0,vect,1)
        r=ddot(nq(nblock),q(1,nqn1,nblock),1,vect,1)
        rr(nfqcy,j)=r*fc
       else
        rr(nfqcy,j)=0.d0
       endif       
      enddo
c       call dspmv('U',nq(nblock),1.d0,h0(1,nblock),q(1,nqn1,nblock),
c     .            1,0.d0,vect,1)
c       r=ddot(nq(nblock),q(1,nqn1,nblock),1,vect,1)
c       write(iout,198) 100.0*r/eig(nqn1,nblock)
  307 pom=cutoff*eig(nqn1,nblock)
      ncut=0
      do j=1,nz(nblock)
       r=abs(rr(nfqcy,j))
       if(r.ge.pom) then
        ncut=ncut+1
        index(ncut)=j
       endif
      enddo
  320 change=.false.
      do j=1,ncut-1
       j1=j+1
       idx1=index(j)
       idx2=index(j1)
       if(abs(rr(nfqcy,idx1)).lt.abs(rr(nfqcy,idx2))) then
        change=.true.
        index(j)=idx2
        index(j1)=idx1
       endif
      enddo
      if(change) go to 320
c************Include force constant descriptions*********************
c******************(if ".force" file exist)**************************
      open(unit=29,file=name(1:l_name)//'.force',err=98,status='old')
      j=0
   91 j=j+1
      read(29,110,end=92) jj,fname(jj)
      if(j.ne.jj) then
       print *,'Error in '//name(1:l_name)//'.force file'
       print *,'Row ',j,' reads ',jj,fname(jj)
       goto 98
      endif
      goto 91
   92 write(iout,111)(fname(nfo(nblock,index(j))),
     . 100.0*rr(nfqcy,index(j))/eig(nqn1,nblock),j=1,ncut)
      close(unit=29)
      goto 99
   98 write(iout,199)(nfo(nblock,index(j)),
     . 100.0*rr(nfqcy,index(j))/eig(nqn1,nblock),j=1,ncut)
   99 if(iout.eq.8) then
       write(10) fq(nblock,nfqcy),sqrt(abs(eig(nqn1,nblock))/factor)
       write(10) (rr(nfqcy,j), j=1,nz(nblock))
      endif
      return
  110 format(i3,a35)
  111 format(2(a35,1x,f4.0))
c********************F matrix PED***********************************
   20 call dgemv('N',nrw(nblock),nq(nblock),1.d0,w(1,1,nblock),nfc,
     .           q(1,nqn1,nblock),1,0.d0,vect,1)
      do l=1,nrw(nblock)
       do k=1,l
        rr(k,l)=0.d0
       enddo
      enddo
      do j=nzz+1,noz
       nnr=nr(nblock,j)
       nnc=nc(nblock,j)
       gkl=z(nblock,j)*fi(nfo(nblock,j))
       r=vect(nnr)*vect(nnc)*gkl/eig(nqn1,nblock)
       if(nnr.ne.nnc) r=r+r
       rr(nnr,nnc)=rr(nnr,nnc)+r
      enddo
      low=0
      do k=1,nz(nblock)
       fc=fi(nfo(nblock,k))
       if(fc.ne.0.d0) then
        do j=low+1,numb(nblock,k)
         nnr=nr(nblock,j)
         nnc=nc(nblock,j)
         r=vect(nnr)*vect(nnc)*fc*z(nblock,j)/eig(nqn1,nblock)
         if(nnr.ne.nnc) r=r+r
         rr(nnr,nnc)=rr(nnr,nnc)+r
        enddo
       endif
       low=numb(nblock,k)
      enddo
      do m=1,nrw(nblock)
       indm=0
       do n=m,nrw(nblock)
        if(abs(rr(m,n)).ge.cutoff) then
         indm=indm+1
         index(indm)=n
        endif
       enddo
       if(indm.gt.0) then
        write(iout,197) (m,index(j),1.d2*rr(m,index(j)),j=1,indm)
        if(iout2.gt.0)
     .    write(iout2,197) (m,index(j),1.d2*rr(m,index(j)),j=1,indm)
       endif
      enddo
      if(iout.eq.8) then
       write(10) fq(nblock,nfqcy),sqrt(abs(eig(nqn1,nblock))/factor)
       write(10) ((rr(k,l), k=1,l), l=1,nrw(nblock))
      endif
      if(typ.eq.-3) goto 30
      return
c********************F matrix PED sensitivity********************
   30 write(iout,*) '================ PED sensitivity =============='
      if(iout2.gt.0)
     .write(iout2,'(a,f6.3)') 
     . '============== PED sensitivity ============= delta fi = ',dump
      if(ve) write(18) nfqcy,nrw(nblock),nvar
      do mm=1,nvar
       nmm=ncc(mm)
       if(mm.gt.1) then 
        mm1=mm-1
        nmm1=ncc(mm1)
        fi(nmm1)=fi(nmm1)-dump
       endif
       if(ve) write(*,'(a,i3)') 'Changing force constant #',nmm
       fi(nmm)=fi(nmm)+dump
       call diagh('V',nblock,nfqcy,vect,ve)
       fi0(nmm)=fi(nmm)
       if(mm.gt.1) fi0(nmm1)=fi(nmm1)
       eigenvalue=vect(1)
       call dgemv('N',nrw(nblock),nq(nblock),1.d0,w(1,1,nblock),nfc,
     .            q(1,1,nblock),1,0.d0,vect,1)
       do l=1,nrw(nblock)
        do k=1,l-1
         rr(l,k)=-rr(k,l)
        enddo
        s(l)=-rr(l,l)
       enddo
       do j=nzz+1,noz
        nnr=nr(nblock,j)
        nnc=nc(nblock,j)
        gkl=z(nblock,j)*fi(nfo(nblock,j))
        r=vect(nnr)*vect(nnc)*gkl/eigenvalue
        if(nnr.eq.nnc) then
         s(nnr)=s(nnr)+r
        else
         rr(nnc,nnr)=rr(nnc,nnr)+r+r
        endif
       enddo
       low=0
       do k=1,nz(nblock)
        fc=fi(nfo(nblock,k))
        if(fc.ne.0.d0) then
         do j=low+1,numb(nblock,k)
          nnr=nr(nblock,j)
          nnc=nc(nblock,j)
          r=vect(nnr)*vect(nnc)*fc*z(nblock,j)/eigenvalue
          if(nnr.eq.nnc) then
           s(nnr)=s(nnr)+r
          else
           rr(nnc,nnr)=rr(nnc,nnr)+r+r
          endif
         enddo
        endif
        low=numb(nblock,k)
       enddo
       if(ve) write(18) (s(l), l=1,nrw(nblock))
c       if(ve) write(18) ((rr(l,k), k=1,l-1), l=1,nrw(nblock))
       indiag=0
       do l=1,nrw(nblock)
        if(abs(s(l)).ge.cutsen) then
         indiag=indiag+1
         index(indiag)=l
        endif 
       enddo
       indoff=0
       do l=1,nrw(nblock)    
        do k=1,l-1
         if(abs(rr(l,k)).ge.cutsen) then
          indoff=indoff+1
          index(indiag+indoff)=l
          nrow(indoff)=k
         endif
        enddo
       enddo
       if(indiag.gt.0) then
        if(indiag.le.5) then
         write(iout,250) nmm,
     .        (index(j),index(j),1.d2*s(index(j)),j=1,indiag)
c         if(iout2.gt.0) write(iout2,250) nmm,
c     .        (index(j),index(j),1.d2*s(index(j)),j=1,indiag)
        else
         nend1=5
         write(iout,250) nmm,
     .        (index(j),index(j),1.d2*s(index(j)),j=1,nend1)
         write(iout,251) 
     .        (index(j),index(j),1.d2*s(index(j)),j=nend1+1,indiag)
c         if(iout2.gt.0) then         
c           write(iout2,250) nmm,
c     .        (index(j),index(j),1.d2*s(index(j)),j=1,nend1)
c           write(iout2,251) 
c     .        (index(j),index(j),1.d2*s(index(j)),j=nend1+1,indiag)
c         endif
        endif
        if(iout2.gt.0) write(iout2,252) nmm,
     .        (index(j),index(j),1.d2*s(index(j)),j=1,indiag)
       endif
       if(indoff.gt.0) then
        if(indoff.le.5) then
         write(iout,250) nmm,(nrow(j),index(indiag+j),
     .         1.d2*rr(index(indiag+j),nrow(j)),j=1,indoff)
c         if(iout2.gt.0) write(iout2,250) nmm,(nrow(j),index(indiag+j),
c     .         1.d2*rr(index(indiag+j),nrow(j)),j=1,indoff)
        else
         nend2=5
         write(iout,250) nmm,(nrow(j),index(indiag+j),
     .         1.d2*rr(index(indiag+j),nrow(j)),j=1,nend2)
         write(iout,251) (nrow(j),index(indiag+j),
     .         1.d2*rr(index(indiag+j),nrow(j)),j=nend2+1,indoff)
c         if(iout2.gt.0) then         
c          write(iout2,250) nmm,(nrow(j),index(indiag+j),
c     .         1.d2*rr(index(indiag+j),nrow(j)),j=1,nend2)
c          write(iout2,251) (nrow(j),index(indiag+j),
c     .         1.d2*rr(index(indiag+j),nrow(j)),j=nend2+1,indoff)
c         endif
        endif
        if(iout2.gt.0) write(iout2,252) nmm,(nrow(j),index(indiag+j),
     .         1.d2*rr(index(indiag+j),nrow(j)),j=1,indoff)
       endif
      enddo
      fi(ncc(nvar))=fi(ncc(nvar))-dump
      call diagh('V',nblock,0,vect,ve)
      fi0(ncc(nvar))=fi(ncc(nvar))
      return
  197 format(5(1x,i3,',',i3,'(',f4.0,')'))
  198 format(30x,'PED (fixed f.c. ',f4.0,') : ')
  199 format(8(1x,i3,'(',f4.0,')'))
  250 format(1x,i3,': ',5(1x,i3,',',i3,'(',f4.0,')'))
  251 format(6x,5(1x,i3,',',i3,'(',f4.0,')'),4x)
  252 format(1x,i3,': ',100(1x,i3,',',i3,'(',f6.2,')'))
      end

      subroutine iso_shifts(iout,iout2,nb,nfqcy,nvar,dump,ve)
      implicit real*8(a-h,o-z)
      include 'Sizes'
      logical ve
      common /qmat/ nq(nbl),nz(nbl),nfo(nbl,ndimz),q(ndim,ndim,nbl)
     .       /hmat/ h0(nkdim,nbl),fi(nfc),fi0(nfc)
     .       /pedm/ nzz,noz,rr(nint,nfc),eig(ndim,nbl),fq(nbl,ndim)
     .       /position/ w(nfc,nfc,nbl),vect(nint),nrw(nbl)
     .       /hkmatrices/ nr(nbl,ndimz),nc(nbl,ndimz),numb(nbl,nfc),
     .                    z(nbl,ndimz)
     .       /sigm/ ncc(nfc),index(nfc)
c***************Isotopic shift sensitivity***********************
      write(iout,*) '================ Shift sensitivity ============='
      if(iout2.gt.0)
     .write(iout2,'(a,f6.3)') 
     . '============= Shift sensitivity ============ delta fi = ',dump
      if(ve) write(19) nfqcy,nb-1,nvar
      do mm=1,nvar
       nmm=ncc(mm)
       if(mm.gt.1) then 
        mm1=mm-1
        nmm1=ncc(mm1)
        fi(nmm1)=fi(nmm1)-dump
       endif
       fi(nmm)=fi(nmm)+dump
       call diagh('N',1,nfqcy,vect,ve)
       freq1=sqrt(abs(vect(1))/factor)
       do i=2,nb
        call diagh('N',i,nfqcy,vect,ve)
        vect(i)=sqrt(abs(vect(1))/factor)-freq1
       enddo
       fi0(nmm)=fi(nmm)
       if(mm.gt.1) fi0(nmm1)=fi(nmm1)
       if(nb.le.7) then
        write(iout,260) nmm,(j,vect(j),j=2,nb)
        if(iout2.gt.0) write(iout2,260) nmm,(j,vect(j),j=2,nb)
       else
        nend2=6
        write(iout,260) nmm,(j,vect(j),j=2,nend2)
        write(iout,261) (j,vect(j),j=2,nend2+1,nb)
        if(iout2.gt.0) then         
          write(iout2,260) nmm,(j,vect(j),j=2,nend2)
          write(iout2,261) (j,vect(j),j=2,nend2+1,nb)
        endif
       endif
       if(ve) write(19) (vect(j),j=2,nb)
      enddo
      fi(ncc(nvar))=fi(ncc(nvar))-dump
      do i=1,nb
       call diagh('V',i,0,vect,ve)
      enddo
      fi0(ncc(nvar))=fi(ncc(nvar))
      return
  260 format(1x,i3,': ',6(1x,i2,'(',f7.2,')'))
  261 format(6x,6(1x,i2,'(',f7.2,')'))
      end

      subroutine sig(dump,nblock,nvar,ve,fcs)
      implicit real*8(a-h,o-z)
      include 'Sizes'
      logical change,ve,fcs
      common  /qmat/ nq(nbl),nz(nbl),nfo(nbl,ndimz),q(ndim,ndim,nbl)
     .        /sigm/ ncc(nfc),index(nfc)
     .        /hmat/ h0(nkdim,nbl),fi(nfc),fi0(nfc)
     .        /position/ w(nfc,nfc,nbl),vect(nint),nrw(nbl)
     .        /pedm/ nzz,noz,rr(nint,nfc),eig(ndim,nbl),fq(nbl,ndim)
      do mm=1,nvar
       nmm=ncc(mm)
       if(mm.gt.1) then 
        mm1=mm-1
        nmm1=ncc(mm1)
        fi(nmm1)=fi(nmm1)-dump
       endif
       write(*,'(a,i3)') 'Changing force constant #',nmm
       fi(nmm)=fi(nmm)+dump
       call diagh('N',nblock,0,vect,ve)
       fi0(nmm)=fi(nmm)
       if(mm.gt.1) fi0(nmm1)=fi(nmm1)
       do k=1,nq(nblock)   
        ik=nq(nblock)-k+1
        rr(k,nmm)=sqrt(abs(vect(ik))/factor)-
     .           sqrt(abs(eig(ik,nblock))/factor)
       enddo
      enddo
      fi(ncc(nvar))=fi(ncc(nvar))-dump
      call diagh('V',nblock,0,vect,ve)
      fi0(ncc(nvar))=fi(ncc(nvar))
      end

      subroutine order(fcs,k,ncut,nblock,nvar,ve)
      implicit real*8(a-h,o-z)
      include 'Sizes'
      logical change,ve,fcs
      common  /qmat/ nq(nbl),nz(nbl),nfo(nbl,ndimz),q(ndim,ndim,nbl)
     .        /sigm/ ncc(nfc),index(nfc)
     .        /pedm/ nzz,noz,rr(nint,nfc),eig(ndim,nbl),fq(nbl,ndim)
      if(ve) print *,'  == ORDER ==  Sorting PED''s ...'
      if(fcs) then
       rrmax=0.d0
       ncut=0
       do j=1,nq(nblock)
        r=abs(rr(j,k))
        if(r.gt.rrmax) rrmax=r
       enddo
       pom=cutsig*rrmax
       do j=1,nq(nblock)
        r=abs(rr(j,k))
        if(r.ge.pom) then
         ncut=ncut+1
         index(ncut)=j
        endif
       enddo
   10  change=.false.
       do j=1,ncut-1
        j1=j+1
        idx1=index(j)
        idx2=index(j1)
        if(abs(rr(idx1,k)).lt.abs(rr(idx2,k))) then
         change=.true.
         index(j)=idx2
         index(j1)=idx1
        endif
       enddo
       if(change) go to 10
      else
       rrmax=0.d0
       ncut=0
       do j=1,nvar
        r=abs(rr(k,ncc(j)))
        if(r.gt.rrmax) rrmax=r
       enddo
       pom=cutsig*rrmax
       do j=1,nvar
        r=abs(rr(k,ncc(j)))
        if(r.ge.pom) then
         ncut=ncut+1
         index(ncut)=ncc(j)
        endif
       enddo
   20  change=.false.
       do j=1,ncut-1
        j1=j+1
        idx1=index(j)
        idx2=index(j1)
        if(abs(rr(k,idx1)).lt.abs(rr(k,idx2))) then
         change=.true.
         index(j)=idx2
         index(j1)=idx1
        endif
       enddo
       if(change) go to 20
      endif
      end
