C**********************************************************************
C                 Copyright 1992-1996, Robert Fraczkiewicz            *
C                           All rights reserved.                      *
C**********************************************************************
C This program is free software; you can redistribute it and/or modify*
C it under the terms of the GNU General Public License as published by*
C the Free Software Foundation; either version 1, or (at your option) *
C any later version.                                                  *
C                                                                     *
C This program is distributed in the hope that it will be useful,     *
C but WITHOUT ANY WARRANTY; without even the implied warranty of      *
C MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU    *
C General Public License for more details.                            *
C                                                                     *
C You should  have received a copy of the GNU General Public License  *
C along with this program; if not, write to the Free Software         *
C Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.           *
***********************************************************************
*                                                                     *
*          Program "Molecular Geometry" by Robert Fraczkiewicz        *
*                    Oct, 92, version 1.1                             *
*                                                                     *
***********************************************************************
* Program calculates Cartesian coordinates using Gaussian-style input *
* of the internal molecular geometry. First four atoms define the     *
* coordinate system.                                                  *
***********************************************************************
*                                                                     *
*     INPUT FILES:                                                    *
*                                                                     *
*     ".gein" - main input file containing molecular geometry in the  *
*               Gaussian style format                                 *
*                                                                     *
***********************************************************************
*                                                                     *
*     OUTPUT FILES:                                                   *
*                                                                     *
*     ".xmat" - Cartesian coordinate in GMAT4 format (X-matrix)       *
*     ".xyz" - Cartesian coordinates in XMol format                   *
*                                                                     *
***********************************************************************
*                                                                     *
*     DIMENSIONS:                                                     *
*                                                                     *
*     n1,n2,n3,r,alfa,tor,atom       - NATOM                          *
*     a,b                            - 3                              *
*     aa                             - 3x3                            *
*     x                              - 3xNATOM                        *
*                                                                     *
***********************************************************************

      program geom
      implicit real*8(a-h,o-z)
      integer natom
      parameter (pi=3.1415926535897932384626433, natom=5000)
      character*14 name,cr
      character*4 atom(natom)
      dimension n1(natom),n2(natom),n3(natom),r(natom),alfa(natom),
     .tor(natom),x(3,natom),a(3),b(3),aa(3,3)
c**********************************************************************
      if(iargc().lt.1) then 
       print *
       print *,' /--------------------------------------------------\ '
       print *,'|                                                    |'
       print *,'|                     G  E  O  M                     |'
       print *,'|                                                    |'
       print *,'|                     version 1.1                    |'
       print *,'|                                                    |'
       print *,'|         Generation of Cartesian Coordinates        |'
       print *,'|                                                    |'
       print *,'|                           by Robert Fraczkiewicz   |'
       print *,'|                      University of Houston, 1992   |'
       print *,' \--------------------------------------------------/ '
       print *
       print *,'______________________________________________________'
       print *
       print *,'SYNOPSIS: '
       print *
       print *,'      geom filename'
       print *
       print *,'PARAMETERS: '
       print *
       print *,'      filename - core name of the input and output'
       print *,'                 files. Its length cannot exceed 14'
       print *,'                 characters. Appropriate extensions'
       print *,'                 are added by default.'
       print *
       print *,'DESCRIPTION:'
       print *
       print *,'      Program calculates atomic Cartesian coordinates.'
       print *
       print *,'______________________________________________________'
       stop 7
      endif
      narg=1
      call getarg(narg,name)
      l_name=namlen(name)
      dr=pi/1.8d2
c**********Start reading internal coordinates matrix******************
c****************Initialize coordinate system*************************
      open(unit=7, file=name(1:l_name)//'.gein', err=9990, 
     .     status='old') 
      read(7,*) atom(1)
      read(7,*) atom(2),n1(2),r(2)
      read(7,*) atom(3),n1(3),r(3),n2(3),alfa(3)
      do i=1,3
       x(i,1)=0.0
       x(i,2)=0.0
      enddo
      x(3,2)=r(2)
      x(2,3)=0.0
      if(n1(3).eq.2) then
       x(3,3)=r(2)+r(3)*sin(dr*(alfa(3)-90.0))
       x(1,3)=r(3)*cos(dr*(alfa(3)-90.0))
      else
       if(n1(3).eq.1) then
        x(3,3)=r(3)*cos(dr*alfa(3))
        x(1,3)=r(3)*sin(dr*alfa(3))
       else
        print *,' = ERROR: check bond for atom #3 ='
        stop 3
       endif
      endif
      n=4
c**********Finish reading internal coordinates************************
   10 read(7,*,err=20) atom(n),n1(n),r(n),n2(n),alfa(n),n3(n),tor(n)
      if(n1(n).ge.n.or.n2(n).ge.n.or.n3(n).ge.n) then
       print *,' = ERROR: check input data for atom ',n,' = '
       stop 4
      endif
      do i=1,3
       a(i)=x(i,n2(n))-x(i,n1(n))
       b(i)=x(i,n3(n))-x(i,n2(n))
      enddo
      axy=a(1)*a(1)+a(2)*a(2)
      axyz=axy+a(3)*a(3)
      axy=sqrt(axy)
      axyz=sqrt(axyz)
      if(axy.eq.0) then
       c1=1.0
       s1=0.0
      else
       c1=a(1)/axy
       s1=a(2)/axy
      endif
      if(axyz.eq.0) then
       print *,' = ERROR: wrong data for ',atom(n),' , n=',n
       print *,'   Check distance between ',atom(n2(n)),' and ',
     1atom(n1(n)),' = '
       stop 5
      endif
      c2=a(3)/axyz
      s2=axy/axyz
      aa(1,1)=c1*c2
      aa(1,2)=s1*c2
      aa(1,3)=-s2
      aa(2,1)=-s1
      aa(2,2)=c1
      aa(2,3)=0.0
      aa(3,1)=c1*s2
      aa(3,2)=s1*s2
      aa(3,3)=c2
      do j=1,2
       a(j)=0.0
       do i=1,3
        a(j)=a(j)+aa(j,i)*b(i)
       enddo
      enddo
      axy=sqrt(a(1)*a(1)+a(2)*a(2))
      c1=a(1)/axy
      s1=a(2)/axy
      s=r(n)*sin(dr*alfa(n))
      cc1=s*cos(-dr*tor(n))
      cc2=s*sin(-dr*tor(n))
      b(1)=c1*cc1-s1*cc2
      b(2)=s1*cc1+c1*cc2
      b(3)=r(n)*cos(dr*alfa(n))      
      do i=1,3
       x(i,n)=x(i,n1(n))
       do j=1,3
      	x(i,n)=x(i,n)+aa(j,i)*b(j)
       enddo
      enddo
      n=n+1
      go to 10
   20 n=n-1
      close(unit=7)
  999 nato=0
      open(unit=8, file=name(1:l_name)//'.xmat', status='unknown')
      do i=1,n
       if(atom(i)(1:1).ne.'x'.and.atom(i)(1:1).ne.'X') then
        nato=nato+1
        write(8,30) atom(i),(x(j,i), j=1,3)
       endif
      enddo
      close(unit=8)
      open(unit=9, file=name(1:l_name)//'.xyz', status='unknown')
      write(9,*) nato
      write(9,'(a14)') name
      do i=1,n
       if(atom(i)(1:1).ne.'x'.and.atom(i)(1:1).ne.'X') then
        l_atom=nsymbol(atom(i))
        write(9,30) atom(i)(1:l_atom),(x(j,i), j=1,3)
       endif
      enddo
      close(unit=9)
   30 format(1x,a4,3(2x,f14.10))
      stop 1
c  ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 9990 print *
      print *,'Input operation unsuccesful. Check the name and format'
      print *,'of the data file. Don"t input the filename extension !'
      stop 2
c  ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      end

      integer function namlen(stri)
      implicit real*8(a-h,o-z)
      character*14 stri
      do i=1,len(stri)
       if(stri(i:i).eq.' ') goto 2
      enddo
    2 namlen=i-1
      return
      end

      integer function nsymbol(stri)
      implicit real*8(a-h,o-z)
      character*4 stri
      do i=1,len(stri)
       if(stri(i:i).eq.'1') goto 2
       if(stri(i:i).eq.'2') goto 2
       if(stri(i:i).eq.'3') goto 2
       if(stri(i:i).eq.'4') goto 2
       if(stri(i:i).eq.'5') goto 2
       if(stri(i:i).eq.'6') goto 2
       if(stri(i:i).eq.'7') goto 2
       if(stri(i:i).eq.'8') goto 2
       if(stri(i:i).eq.'9') goto 2
       if(stri(i:i).eq.'0') goto 2
      enddo
    2 nsymbol=i-1
      return
      end
