C**********************************************************************
C                 Copyright 1992-1996, Robert Fraczkiewicz            *
C                           All rights reserved.                      *
C**********************************************************************
C This program is free software; you can redistribute it and/or modify*
C it under the terms of the GNU General Public License as published by*
C the Free Software Foundation; either version 1, or (at your option) *
C any later version.                                                  *
C                                                                     *
C This program is distributed in the hope that it will be useful,     *
C but WITHOUT ANY WARRANTY; without even the implied warranty of      *
C MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU    *
C General Public License for more details.                            *
C                                                                     *
C You should  have received a copy of the GNU General Public License  *
C along with this program; if not, write to the Free Software         *
C Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.           *
c**********************************************************************
c                                                                     *
c          Program "G-Matrix Generator" by Robert Fraczkiewicz        *
c                       Jul 18, 94, version 4.5                       *
c                                                                     *
c**********************************************************************
c Modernized and expanded version of Schachtschneider's program GMAT  *
c (Schachtschneider, J.H. Technical Report No. 231-64, Project No.    *
c  31450, Shell Development Co., Emeryville, CA, 1964)                *
c This program evaluates G and B matrix elements for up to 600        *
c internal coordinates for molecules containing up to 200 atoms and   *
c for five types of internal coordinates:                             *
c        1. bond stretching,                                          *
c        2. valence angle bending,                                    *
c        3. out of plane wagging,                                     *
c        4. torsion,                                                  *
c        5. linear valence bending.                                   *
c G-matrix is symmetrized.                                            *
c Input data are the masses, Cartesian coordinates of the atoms in an *
c arbitrary reference frame, internal coordinate definitions, and a   *
c U-matrix. Additionally the program calculates a Beta-matrix contai- *
c ning Eckart-Sayvetz conditions (input for DT2).                     *
c**********************************************************************
c                                                                     *
c     INPUT FILES:                                                    *
c                                                                     *
c     ".gmin" - main input file containing internal coordinate defi-  *
c               nitions, atomic masses and the name of U-matrix file  *
c     ".umat" - U-matrix (user-defined)                               *
c     ".xmat" - X-matrix (output of GEOM)                             *
c                                                                     *
c**********************************************************************
c                                                                     *
c     OUTPUT FILES:                                                   *
c                                                                     *
c     ".gmout" - main output containing B, G and U matrices in ASCII  *
c                format. Not essential for other programs.            *
c     ".gmat" - G-matrices in binary format. Input for other programs *
c               in the package.                                       *
c     ".bmat" - B-matrix in binary format. Input for DT2.             *
c     ".beta" - beta-matrices in ASCII format. Input for DT2.         *
c                                                                     *
c**********************************************************************
      program gmat4
      implicit real*8(a-h,o-z)
      parameter (pi=3.1415926535897932384626433)
      character*14 name,qs,cr
      character*4 atom(100)
      data qs /'?'/
      logical iquit,uskip,xerror
      common x(3,200),b(3500),nr(3500),nc(3500)
      dimension  nrg(9000),ncg(9000),dg(9000),nru(2500),ncu(2500),
     1 du(2500),wt(200),w(600),bb(600),g(600),gu(600),nrs(3500),
     2 ncs(3500),gs(3500),nrow(4),ncol(4),dat(4),ne(4),ncod(4),ni(4),
     3 nj(4),nk(4),nl(4),nx(4),ny(4),nb(20),u(600),ug(600),info(36),
     4 news(18)
      equivalence (g(1),ug(1),u(1)),(bb(1),gu(1))
c**********************************************************************
      if(iargc().eq.0) then
       print *
       print *,' /--------------------------------------------------\ '
       print *,'|    GGGGG   M     M     A     TTTTTTT    4          |'
       print *,'|   G     G  MM   MM    A A    T  T  T   4           |'
       print *,'|   G     G  M M M M   A   A      T     4   4        |'
       print *,'|   G        M  M  M  A     A     T    4444444       |'
       print *,'|   G   GGG  M     M  AAAAAAA     T         4        |'
       print *,'|   G     G  M     M  A     A     T         4        |'
       print *,'|    GGGGG   M     M  A     A     T         4        |'
       print *,'|                                                    |'
       print *,'|                     version 4.5                    |'
       print *,'|                                                    |'
       print *,'|                 G-matrix Generator                 |'
       print *,'|                                                    |'
       print *,'|                                                    |'
       print *,'|                           by Robert Fraczkiewicz   |'
       print *,'|                      University of Houston, 1992-4 |'
       print *,' \--------------------------------------------------/ '
       print *
       print *,'______________________________________________________'
       print *
       print *,'SYNOPSIS: '
       print *
       print *,'      gmat4 filename'
       print *
       print *,'PARAMETERS: '
       print *
       print *,'      filename - core name of the input and output'
       print *,'                 files. Its length cannot exceed 14'
       print *,'                 characters. Appropriate extensions'
       print *,'                 are added by default.'
       print *
       print *,'DESCRIPTION:'
       print *
       print *,'      Program generates G, B and beta matrices. Type '
       print *,'      "gmat4 \?" to see the format of the input file.'
       print *,'      Binary output files: filename_#_%.gmat, '
       print *,'      filename_%.beta and filename.bmat are the inputs'
       print *,'      for other NCA programs. # denotes the number of'
       print *,'      symmetry block; % is the number of isotopic'
       print *,'      molecule in filename.gmin.'
       print *
       print *,'______________________________________________________'
       stop 5
      endif
      call getarg(1,name)
      l_name=namlen(name)
      if (name.eq.qs) goto 2000
      open(unit=7, file=name(1:l_name)//'.gmin', err=2990, 
     .     status='old') 
      open(unit=8, file=name(1:l_name)//'.gmout', status='unknown')
c*******************************************************************
c     Read problem control. Free format                            *
c       1. noprob, the problem number,                             *
c       2. noat, the number of atoms in the molecule, < or = 200   *
c       3. nq, the number of internal coordinates, < or = 600      *
c       4. niso=0, or 1. niso is set equal to 1 if the G-matrix    *
c               of more than one isotopic molecule is desired.     *
c               niso=0 if only one.                                *
c       5. ifb, B-matrix output options. if ifb=0 the B-matrix is  *
c               written (.gmout file) but not stored.              *
c               if ifb=1 the B-matrix is both written and          *
c               stored in the file name.bmat                       *
c*******************************************************************
   93 read(7,*) noprob,noat,nq,niso,ifb
c********************************************************************
c     Input data. The input follows the control line in the order:  *
c         1. two problem information lines, format(18a4) containing *
c            alpha-numeric information identififying the problem.   *
c            this information is reproduced with the output.        *
c            First three columns of each line must be blank.        *
c         2. name of the X-matrix file                              *
c         3. the X-matrix, format(4(2i3,f12.6)),giving the carte-   *
c            sian coordinates of the atoms. (Read from .xmat file)  *
c         4. the internal coordinate vectors, format(3(8i3)),       *
c            defining the internal coordinates and the cartesians   *
c            format(3f12.6), for the a- points giving direction     *
c            to linear bending coordinates.                         *
c         5. isotope control line, for molecule 1, format(4i3,17a4).*
c         6. masses for molecule 1, format(6f12.6).                 *
c         7. U-matrix file name (.umat file)                        *
c     If niso=1 items 5,6 are then repeated as necessary for        *
c     each isotopic molecule.                                       *
c********************************************************************      
c
c********************************************************
c     Read two problem information lines. Format(18a4)  *
c********************************************************
      read(7,14)(info(i),i=1,36)
   14 format(18a4)
      write(8,50)noprob,(info(i),i=1,36),noat,nq
   50 format(1h1,31hg matrix evaluation for problem,i9,1h.//
     *       80(1h*)/2(2h *,18a4,5x,1h*/),80(1h*)/
     *       1h0,i4,7h atoms,,i10,22h internal coordinates./)
   94 na=3*noat
      joker=0
      iquit=.false.
      uskip=.false.
      do 102 i=1,3
      do 102 j=1,noat
  102 x(i,j)=0.0
      ncard=0
c**********************************************************************
c     Read X-matrix from the .xmat file                               *
c     Name core must be supplied from .gmin input file after two text *
c     lines and cannot exceed 14 characters.                          *  
c     Format:                                                         *
c      comment (80 characters)                                        *
c      atom (4 char.), x, y, z                                        *
c     format(a4,3(2x,zf14.10))                                        *
c     Program geom gives output in this form.                         *
c**********************************************************************
      read(7,'(a14)') qs
      l_qs=namlen(qs)
      open(unit=4, file=qs(1:l_qs)//'.xmat', err=2991, status='old') 
      i=0
  104 i=i+1
      read(4,939,err=112) atom(i),(x(j,i), j=1,3)
      goto 104
  939 format(1x,a4,3(2x,f14.10))
  112 close(unit=4)
      i=i-1
      if(i.gt.noat.or.i.eq.1) goto 600
  113 write(8,930)
  930 format(1h-,31hx matrix, cartesian coordinates/32(1h*)/)
      do 114 k=1,noat,5
      l=min0(noat,(4+k))
      write(8,933)(i,i=k,l)
  933 format(1x,'atom',i11,4i16)
      write(8,935)(x(1,i),i=k,l)
  935 format(5(2x,f14.10))
      write(8,935)(x(2,i),i=k,l)
      write(8,935)(x(3,i),i=k,l)
  114 continue
c****************************************************************
c     Check that inter-atomic distances are reasonable. Comment *
c     if any of them is shorter than 1 angstrom                 *
c****************************************************************
      xerror=.false.
      do 116 i=2,noat
      l=i-1
      do 115 j=1,l
      dsq=(x(1,i)-x(1,j))*(x(1,i)-x(1,j))
      dsq=dsq+(x(2,i)-x(2,j))*(x(2,i)-x(2,j))
      dsq=dsq+(x(3,i)-x(3,j))*(x(3,i)-x(3,j))
      if(dsq.gt.1.0) go to 115
      dsq=dsqrt(dsq)
      xerror=.true.
      write(8,900)i,j,dsq
  900 format(15h warning! atoms,i3,4h and,i3,10h have only,f8.3,
     *       25h angstroms between them.<,58(1h=),7hwarning)
  115 continue
  116 continue
      if(.not.xerror)write(8,910)
  910 format(1x,'Atom distance check completed, no distances less ',
     . 'than one angstrom found.'/)
      nob=0
      noint=0
      write(8,51)
   51 format(36h-   internal coordinate definitions./
     *       43h0   no   code   i    j    k    l    ix   jx/)
      ncard=0
c*********************************************************************
c     Read internal coordinate vectors. Format(24i3)                 *
c     The "vector" describing the internal coordinate is made up of  *
c     8 integers called ni,ncod,n1,n2,n3,n4,n5,n6.                   *
c        ni, gives the number assigned to the internal coordinate,   *
c        ncod, is a code identifinig the type of internal coordinate,*
c        n1 thru n6 give the numbers of the atoms defining the       *
c        internal coordinate.                                        *
c                                                                    *
c        type               code   n1   n2   n3   n4   n5   n6       *
c     stretching              1    i    j                            *
c     bending                 2    i    j    k         ix   jx       *
c     out of plane            3    i    j    k    l    ix   jx       *
c     torsion                 4    i    j    k    l    ix   jx       *
c     linear bend             5    no2  i    j    k    ix   jx  pair *
c     linear bend             6    i    j    k         ix   jx       *
c                                                                    *
c     The internal coordinates must be defined in the order of ni,   *
c     i.e., ni=1,2,3,....,nq. ix and jx  give atom numbers defining  *
c     a bond distance by which the angular coordinates are weighted. *
c     If ix=jx=0, the angles are not weighted.                       *
c     Immediately following each line defining one or more linear    *
c     bending coordinates (ncod=5 or 6) the x,y,z coordinates of the *
c     points 'a' giving direction to linear bending coordinates      *
c     must be read in. These are typed in three 12-column fields.    *
c     if ni=0, the b-matrix elements for the coordinate are added to *
c     those of the previousely defined coordinate. In this way a     *
c     coordinate may be defined as the sum of two or more basic      *
c     internal coordinates. This is useful for defining torsional    *
c     coordinates.                                                   *
c     The last internal coordinate definition line or 'a'-line is    *
c     followed by a line with-02 in columns 1 to 3.                  *
c*********************************************************************
  118 read(7,18,err=2993)(ne(l),ncod(l),ni(l),nj(l),nk(l),nl(l),nx(l),
     .ny(l),l=1,3)
   18 format(24i3)
      ncard=ncard+1
      do 180 l=1,3
      if(ne(l).eq.(-02)) go to 181
      if(ne(l))605,124,122
  122 noint=noint+1
  124 if(ne(l).gt.nq) go to 605
      if(ncod(l))605,180,125
  125 if(ncod(l).gt.7) go to 605
      mx=ncod(l)
      n1=ni(l)
      n2=nj(l)
      n3=nk(l)
      n4=nl(l)
      n5=nx(l)
      n6=ny(l)
      write(8,53)noint,mx,n1,n2,n3,n4,n5,n6
   53 format(1x,2i5,i6,5i5)
      go to(130,140,150,160,170,172),mx
c*****************************
c     stretching subroutine  *
c*****************************
  130 call bost(noat,noint,n1,n2,n3,n4,n5,n6,nob,joker)
      go to 174
c*****************************
c     bending subroutine     *
c*****************************
  140 call bend(noat,noint,n1,n2,n3,n4,n5,n6,nob,joker)
      go to 174
c****************************************
c     out of plane wagging subroutine   *
c****************************************
  150 call opla(noat,noint,n1,n2,n3,n4,n5,n6,nob,joker)
      go to 174
c*****************************
c     torsion subroutine     *
c*****************************
  160 call tors(noat,noint,n1,n2,n3,n4,n5,n6,nob,joker,nbc)
      go to 174
  170 no2=noint+1
c****************************************
c     linear bending subroutine         *
c****************************************
      call libe(noat,noint,n1,n2,n3,n4,n5,n6,nob,joker,no2)
      noint=noint+1
      go to 174
  172 call libe(noat,noint,n1,n2,n3,n4,n5,n6,nob,joker,no2)
      go to 174
  174 if(joker)180,180,605
  605 write(8,982) ne(l),ncod(l),ni(l),nj(l),nk(l),nl(l),nx(l),ny(l),
     *joker
  982 format(1x,'Error while processing this internal coordinate: ',
     *8i3,/,' This is a class ',i2,' blunder.',/,
     *' Check the internal coordinate definitions (.gmin file) 
     *and Cartesian coordinates (.xmat). Are all the atoms in order?')
      iquit=.true.
      joker=0
  180 continue
      go to 118
  181 if(noint.eq.nq) go to 182
      write(8,988)noint,nq,noint
  988 format(9h0warning.,i4,40h internal coordinates have been defined,
     *       ,i4,35h were expected. nq will be reset to,i4,2h.<,
     *       18(1h=),7hwarning)
      nq=noint
  182 nib=nob+1
      nr(nib)=-5
      nc(nib)=0
      b(nib)=0.0
c*********************************
c     output b-matrix for ifb=1  *
c*********************************
      if(ifb)200,195,190
  190 open(unit=10,file=name(1:l_name)//'.bmat',status='unknown',form
     .='unformatted')
      write(10) nob,3*noat
      write(10) (nr(k),nc(k),b(k),k=1,nob)
      close(unit=10)
c************************
c     write b-matrix    *
c************************
  195 write(8,55)noprob,(info(i),i=1,36)
   55 format(1h1,29hb matrix elements for problem,i9,1h.//
     *      80(1h*)/2(2h *,18a4,5x,1h*/),80(1h*)///)
      write(8,25)(nr(k),nc(k),b(k),k=1,nob)
   25 format(4(2x,2i3,f12.6))
      write(8,59)nob
   59 format(1h0,i5,19h non-zero elements./)
      if(.not.iquit) go to 200
      write(8,990)noprob
      print *,'internal coordinate error !'
  990 format(1x,'note. b matrix for problem',i10,
     *'incomplete due to internal coordinate error. <',35(1h=),
     *'warning')
      stop 8
  200 if(niso.le.0) go to 205
      rewind 1
      write(1)(nr(k),nc(k),b(k),k=1,nib)
  205 nmol=0
c***********************************************************************
c     read isotope control line, format(4i3,18a4), containing the      *
c     following information:                                           *
c        1. ifu=-1, 0 or 1, u-matrix read option control.              *
c           ifu=1, the u-matrix is read in and g symmetrized;          *
c                  in this case give the u-matrix filename !!!         *
c           ifu=0, no u-matrix is read and only unsymmetrized          *
c                  g is obtained.                                      *
c           ifu=-1, no u-matrix is read and the u-matrix from          *
c                  previous isotopic molecule is used to symmetrize g. *
c           ifu=-2, gmat4 exits.                                       *
c     the remainder of the line may be used for alpha-numeric infor-   *
c     mation to identify the isotopic molecule.                        *
c***********************************************************************
  210 read(7,20,err=999) ifu,(news(i),i=1,18)
   20 format(i2,18a4)
      if(ifu.le.-2) goto 999
      nmol=nmol+1
      if(uskip) uskip=(ifu.ne.1)
c***********************************************************************
c     Read masses.                                                     *
c     Masses are entered in order of the atom numbers, i.e., in order  *
c     of noat.                                                         *
c***********************************************************************
      read(7,*)(wt(l),l=1,noat)
   22 format(4(f12.6,','))
      write(8,67)noprob,nmol,(info(i),i=1,36),(news(i),i=1,18),
     1(wt(j),j=1,noat)
   67 format(1h1,29hg matrix elements for problem,i9,10h, molecule,
     *        i3,1h.//80(1h*)/2(2h *,18a4,5x,1h*/),
     *       2(1h*),77(1h-),1h*/2h *,18a4,5x,1h*/80(1h*)//
     *       1h-,40hatomic masses used for this molecule are//
     *       (8f10.5)/)
c***********************************************************************
c     write eckart conditions matrix (beta)                            *
c***********************************************************************
      if(nmol.le.9) then
       cr='_'//char(nmol+48)
      else
       nin1=int(float(nmol)/10.0)
       nin2=nmol-10*nin1
       cr='_'//char(nin1+48)//char(nin2+48)
      endif
      l_cr=namlen(cr)
      nbeta=0
      print *, 'Writing '//name(1:l_name)//cr(1:l_cr)//'.beta'
      open(unit=10, file=name(1:l_name)//cr(1:l_cr)//'.beta',
     .     status='unknown')
      do j=1,noat
       j3=3*(j-1)
       do i=1,3 
        nbeta=nbeta+1      
        nru(nbeta)=i
        ncu(nbeta)=j3+i
        du(nbeta)=wt(j)
       enddo
       nbeta=nbeta+1
       nru(nbeta)=4
       ncu(nbeta)=j3+2
       du(nbeta)=-wt(j)*x(3,j)
       nbeta=nbeta+1
       nru(nbeta)=4
       ncu(nbeta)=j3+3
       du(nbeta)=wt(j)*x(2,j)
       nbeta=nbeta+1
       nru(nbeta)=5
       ncu(nbeta)=j3+1
       du(nbeta)=wt(j)*x(3,j)
       nbeta=nbeta+1
       nru(nbeta)=5
       ncu(nbeta)=j3+3
       du(nbeta)=-wt(j)*x(1,j)
       nbeta=nbeta+1
       nru(nbeta)=6
       ncu(nbeta)=j3+1
       du(nbeta)=-wt(j)*x(2,j)
       nbeta=nbeta+1
       nru(nbeta)=6
       ncu(nbeta)=j3+2
       du(nbeta)=wt(j)*x(1,j)
      enddo
      nbeta=nbeta+1
      nru(nbeta)=-1
      ncu(nbeta)=0
      du(nbeta)=0.0
      write(10,57) (nru(k),ncu(k),du(k),k=1,nbeta)
      close(unit=10)
   57 format(4(2i3,g15.8))
      ng=0
      nt=1
      nub=1
      do 216 l=1,noat
      do 216 m=1,3
      k=3*(l-1)+m
      w(k)=1.0/wt(l)
  216 bb(k)=0.0
      do 218 i=1,nq
  218 g(i)=0.0
      if(niso)220,220,219
  219 rewind 1
      read(1)(nr(k),nc(k),b(k),k=1,nib)
  220 do 250 k=1,nib
  221 if(nr(k)-nt)222,240,222
  222 do 226 l=nub,nob
      i=nr(l)
      j=nc(l)
  226 g(i)=g(i)+bb(j)*w(j)*b(l)
      do 232 i=nt,nq
      if(dabs(g(i))-0.5d-06)232,232,229
  229 ng=ng+1
      nrg(ng)=nt
      ncg(ng)=i
      dg(ng)=g(i)
  232 continue
      if(nr(k))252,252,234
  234 nt=nt+1
      nub=k
      do 238 i=nt,nq
  238 g(i)=0.0
      do 239 i=1,na
  239 bb(i)=0.0
      go to 221
  240 jx=nc(k)
  250 bb(jx)=bb(jx)+b(k)
  252 nog=ng+1
      nrg(nog)=-1
      ncg(nog)=0
      dg(nog)=0.0
c*************************************************
c     output unsymmetrized g-matrix              *
c*************************************************
      write(8,59)
      write(8,25)(nrg(l),ncg(l),dg(l),l=1,ng)
      write(8,59)ng
      if(nmol.eq.1) then
       open(unit=10,file=name(1:l_name)//'_1.gmat',
     . status='unknown', form='unformatted')
       write(10) ng
       write(10)(nrg(l),ncg(l),dg(l),l=1,ng)
       close(unit=10)
      endif
  260 if(ifu)261,210,263
  261 if(uskip) go to 210
      rewind 2
      read(2)(nru(k),ncu(k),du(k),k=1,nou)
      go to 282
  263 nou=0
      jx=1
      nsc=1
      ncard=0
c******************************************************************
c     read u-matrix                                               *
c     file name must be included in *.gmin input file after       *
c     atomic masses                                               *
c     file structure :                                            *
c     nint ns nsb                                                 *
c     nb(1) nb(2) ... nb(nsb)                                     *
c     nel ncol dat ncol dat ..... ncol dat                        *
c     (..................................)                        *
c     nel ncol dat ncol dat ..... ncol dat                        *
c                                                                 *
c     elements must be entered in row order, i.e., row 1,2,3,...  *
c     etc. nel is number of nonzero elements in give row.         *
c     the u-matrix need not to be normalized since the program    *
c     carries out a normalization.                                *
c     nsb means the number of factored blocks + 1.                *
c     nb vector contains symmetry block information,              *
c     the row number for each beginning of each factored block.   *
c     a total of nsb numbers, the last being ns + 1.              *
c******************************************************************
      read(7,'(a14)') qs
      open(unit=3, file=qs(1:l_qs)//'.umat', err=2992, status='old') 
      read(3,*) nnnn,ns,nsb
      read(3,*)(nb(i),i=1,nsb)
      do i=1,ns
       dsq=0.d0
       read(3,*) nzr,(ncu(l),du(l),l=nou+1,nou+nzr)
       nozr=nou+nzr
       no1=nou+1
       do j=no1,nozr
        nru(j)=i
        dsq=dsq+du(j)*du(j)
        if(ncu(j).le.0.or.nru(j).le.0) go to 615
        if(ncu(j).gt.nq.or.nru(j).gt.ns) go to 615
       enddo
       dsq=1.d0/dsqrt(dsq)
       do j=no1,nozr
        du(j)=dsq*du(j)
       enddo
       nou=nozr
      enddo
      close(unit=3)
      nou=nou+1
      nru(nou)=-3
      ncu(nou)=0
      du(nou)=0.0
      write(8,952)noprob,nmol,(info(i),i=1,36),(news(i),i=1,18)
  952 format (1x,'Normalised u matrix elements for problem',i9,
     *        ', molecule',i3,1h.//80(1h*)/2(2h *,18a4,5x,1h*,/),
     *        2h *,77(1h-),1h*/2h *,18a4,5x,1h*/80(1h*)///)
      write(8,25)(nru(i),ncu(i),du(i),i=1,nou)
      write(8,59)nou
      if(niso)282,282,280
  280 rewind 2
      write(2)(nru(i),ncu(i),du(i),i=1,nou)
  282 nu=nou-1
      jl=1
      jx=2
      nut=1
      nub=1
      ix=1
      joker=0
c**********************************************
c     symmetrize, write and output g-matrix   *
c**********************************************
      write(8,61)noprob,nmol,(info(i),i=1,36),(news(i),i=1,18)
   61 format(1h1,41hsymmetrized g matrix elements for problem,i9,
     *       10h, molecule,i3,1h.//80(1h*)/2(2h *,18a4,5x,1h*/),
     *       2h *,77(1h-),1h*/2h *,18a4,5x,1h*/80(1h*)///)
  290 write(8,62)jl
   62 format(1h0,2x,13h block number,i3,1h.)
      if(jl.le.9.) then
       cr='_'//char(jl+48)
      else
       nin1=int(float(jl)/10.0)
       nin2=jl-10*nin1
       cr='_'//char(nin1+48)//char(nin2+48)
      endif
      l_cr=namlen(cr)
      if(nmol.le.9) then
       cr=cr(1:l_cr)//'_'//char(nmol+48)
      else
       nin3=int(float(nmol)/10.0)
       nin4=nmol-10*nin3
       cr=cr(1:l_cr)//'_'//char(nin3+48)//char(nin4+48)
      endif
      l_cr=namlen(cr)
      print *,'Writing '//name(1:l_name)//cr(1:l_cr)//'.gmat'
      open(unit=9,file=name(1:l_name)//cr(1:l_cr)//'.gmat',
     .status='unknown',form='unformatted')
      np=nb(jl)
      nt=nb(jx)
      nel=0
  300 do 302 i=1,nq
       gu(i)=0.0
  302 u(i)=0.0
      do 320 k=nub,nou
       if(nru(k)-ix)306,315,306
  306  do 312 l=1,ng
        i=nrg(l)
        j=ncg(l)
        gu(i)=gu(i)+u(j)*dg(l)
        if(i-j)310,312,310
  310   gu(j)=gu(j)+u(i)*dg(l)
  312  continue
       go to 322
  315  ju=ncu(k)
  320 u(ju)=du(k)
      k=nou
  322 nub=k
      do 324 i=ix,ns
  324 ug(i)=0.0
      assign 330 to jak
      do 335 l=nut,nu
       go to jak,(330,332)
  330  if(nru(l)-ix)331,332,331
  331  assign 332 to jak
       nwy=l
  332  i=nru(l)
       j=ncu(l)
  335 ug(i)=ug(i)+du(l)*gu(j)
      nut=nwy
      do 350 j=ix,ns
       if(0.5d-06-dabs(ug(j)))342,350,350
  342  if(nt-j)343,343,344
  343  print 72,ix,j,ug(j)
       write(8,72)ix,j,ug(j)
       joker=joker+1
  344  nel=nel+1
       nrs(nel)=ix-np+1
       ncs(nel)=j-np+1
       gs(nel)=ug(j)
  350 continue
      ix=ix+1
      if(nt-ix)354,354,300
  354 write(9) nel
      write(9)(nrs(i),ncs(i),gs(i),i=1,nel)
      write(8,25)(nrs(i),ncs(i),gs(i),i=1,nel)
      close(unit=9)
      jl=jl+1
      jx=jx+1
      if(nsb-jx)210,290,290
  600 print 80, i
      write(8,80) i
   56 format(4(2i3,f14.10))
   72  format(39h error, g not factoring. element in row,i3,7h column,
     *       i3,10h has value,1pe12.5,1h<,53(1h=),5herror)
   80 format(1x,' error in x matrix input, row ',i3)
      go to 999
  610 print 84,noprob,l,ncard,(nrow(i),ncol(i),dat(i),i=1,4)
      write(8,84)noprob,l,ncard,(nrow(i),ncol(i),dat(i),i=1,4)
   84 format(36h error in b matrix input for problem,i10,7h. field,i2,
     *       8h of card,i4,2h.<,59(1h=),5herror/10x,11h card reads/
     *       20x,4(2i3,f12.6))
      go to 999
  615 print 86,noprob,ncu(j),nru(j),du(j)
      write(8,86)noprob,ncu(j),nru(j),du(j)
   86 format('error in u matrix input for problem ',i3,'. column ',i2,
     *       ' of row ',i4,' element = ',f9.6)
      if(niso.gt.0) go to 625
  620 print 985,noprob,jx,ns
      write(8,985)noprob,jx,ns
  985 format(30h error in u matrix for problem,i10,1h.,i4,
     *       37h symmetry coordinates have been read,,i4,
     *       16h were expected.<,26(1h=),5herror)
      if(niso)999,999,625
  625 uskip=.true.
      go to 210
  999 close(unit=7)
      close(unit=8)
      stop 1
c  ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 2991 print *
      print *,'Input operation unsuccesful. Cannot find x-matrix file'
      stop 3
 2992 print *
      print *,'Input operation unsuccesful. Cannot find u-matrix file'
      stop 4
 2993 print *
      print *,'Error in internal coordinate definitions in .gmin file'
      print *,'The last read field was:'
      print 18,(ne(l),ncod(l),ni(l),nj(l),nk(l),nl(l),nx(l),
     .ny(l),l=1,3)
      stop 5
 2990 print *
      print *,'Input operation unsuccesful. Check the name and format'
      print *,'of the data file. Don"t input filename extension!'
 2000 print *,'Data file format :'
      print *
      print *,'noprob  noat nq niso ifb'
      print *,'(comment line)'
      print *,'(comment line)'
      print *,'Name of x-matrix file'
      print *,'ne  ncod  ni  nj  nk  nl  nx  ny'
      print *,'(..............................)'
      print *,'ne  ncod  ni  nj  nk  nl  nx  ny'
      print *,'-02'
      print *,'ifu (comment)'
      print *,'w(1) w(2) w(3) w(4) ... w(noat)'
      print *,'Name of u-matrix file (only for ifu=1)'
      print *,'ifu (comment)'
      print *,'w(1) w(2) w(3) w(4) ... w(noat)'
      print *,'(..............................)'
      print *,'ifu (comment)'
      print *,'w(1) w(2) w(3) w(4) ... w(noat)'
      print *
      print *,'where : noprob - the problem number'
      print *,'noat - number of atoms, nq - the number of internal'
      print *,'coordinates, niso - number of additional isotopic'
      print *,'substituted molecules, ifb, ifg - b and g matrix output'
      print *,'options, ne...ny - internal coordinate "vectors", ifu -'
      print *,'u matrix input option, w(i) - masses.'
      stop 2
c  ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      end                                                              

      integer function namlen(stri)
      implicit real*8(a-h,o-z)
      character*14 stri
      do i=1,len(stri)
       if(stri(i:i).eq.' ') goto 2
      enddo
    2 namlen=i-1
      return
      end

      subroutine bost (noat,noint,n1,n2,n3,n4,n5,n6,nob,joker)
c****************************************************************
c     this subroutine computes the b matrix elements for a bond *
c     stretch as defined by wilson.                             *
c****************************************************************
      implicit real*8(a-h,o-z)
      common x(3,200),b(3500),nr(3500),nc(3500)
      dimension rij(3)
  100 if(n6)130,101,130
  101 if(n5)130,102,130
  102 if(n4)130,103,130
  103 if(n3)130,104,130
  104 if(noat-n2)130,105,105
  105 if(noat-n1)130,106,106
  106 i=n1
      j=n2
      dijsq=0.0
  109 do m=1,3
       rij(m)=x(m,j)-x(m,i)
       dijsq=dijsq+rij(m)*rij(m)
      enddo
  114 do m=1,3
       nob=nob+1
       nr(nob)=noint
       nc(nob)=3*(i-1)+m
       b(nob)=-rij(m) /sqrt(dijsq)
       nob=nob+1
       nr(nob)=noint
       nc(nob)=3*(j-1)+m
       b(nob)= rij(m) / sqrt(dijsq)
  120 enddo
      go to 132
  130 joker=1
  132 return
      end
c
      subroutine bend (noat,noint,n1,n2,n3,n4,n5,n6,nob,joker)
c******************************************************************
c     this subroutine computes the b matrix elements of a valence *
c     angle bending coordinate as defined by wilson.              *
c      i and k are the numbers of the end atoms.                  *
c      j= the number of the central atom.                         *
c******************************************************************
      implicit real*8(a-h,o-z)
      common x(3,200),b(3500),nr(3500),nc(3500)
      dimension rji(3),rjk(3),rixjx(3),eji(3),ejk(3)
  100 if(noat-n6)150,101,101
  101 if(noat-n5)150,102,102
  102 if(n4)150,103,150
  103 if(noat-n3)150,104,104
  104 if(noat-n2)150,105,105
  105 if(noat-n1)150,106,106
  106 i=n1
  107 j=n2
      k=n3
      ix=n5
      jx=n6
      if(ix)110,110,112
  110 ix=1
      jx=1
  112 djisq=0.0
      djksq=0.0
      dxsq=0.0
  115 do m=1,3
       rji(m)=x(m,i)-x(m,j)
       rjk(m)=x(m,k)-x(m,j)
       rixjx(m)=x(m,jx)-x(m,ix)
       djisq=djisq+rji(m)*rji(m)
       djksq=djksq+rjk(m)*rjk(m)
       dxsq=dxsq+rixjx(m)*rixjx(m)
      enddo
  123 dji=sqrt(djisq)
      djk=sqrt(djksq)
      dx=sqrt(dxsq)
      if(dx.lt.1.d-12) dx=1.0d0
  128 dotj=0.0d0
  129 do 132 m=1,3
      eji(m)=rji(m)/dji
      ejk(m)=rjk(m)/djk
  132 dotj=dotj+eji(m)*ejk(m)
      if(1.0d0-abs(dotj))152,152,134
  134 sinj=sqrt(1.0d0-dotj*dotj)
  136 do 144 m=1,3
       smi=(dx*(dotj*eji(m)-ejk(m)))/ (dji*sinj)
       nob=nob+1
       nr(nob)=noint
       nc(nob)=3*(i-1)+m
       b(nob)=smi
       smk= (dx*(dotj*ejk(m)-eji(m)))/ (djk*sinj)
       nob=nob+1
       nr(nob)=noint
       nc(nob)=3*(k-1)+m
       b(nob)=smk
       sum=smi+smk
       nob=nob+1
       nr(nob)=noint
       nc(nob)=3*(j-1)+m
       b(nob)=-sum
  144 continue
      go to 154
  150 joker=1
      go to 154
  152 joker=2
  154 return
      end
c
      subroutine opla (noat,noint,n1,n2,n3,n4,n5,n6,nob,joker)
c****************************************************************
c     this subroutine computes the b matrix elements for an out *
c     of plane wagging coordinate as defined by wilson.         *
c      i= the end atom                                          *
c      j= the apex atom                                         *
c      k and l = the anchor atoms.                              *
c****************************************************************
      implicit real*8(a-h,o-z)
      common x(3,200),b(3500),nr(3500),nc(3500)
      dimension rji(3),rjk(3),rjl(3),rixjx(3),eji(3),ejl(3),ejk(3),
     1c1(3),c2(3),c3(3)
  100 if(noat-n6)170,101,101
  101 if(noat-n5)170,102,102
  102 if(noat-n4)170,103,103
  103 if(noat-n3)170,104,104
  104 if(noat-n2)170,105,105
  105 if(noat-n1)170,106,106
  106 i=n1
      j=n2
      k=n3
      l=n4
      ix=n5
      jx=n6
      if(ix)110,110,112
  110 ix=1
      jx=1
  112 djisq=0.0d0
      djksq=0.0d0
      djlsq=0.0d0
  115 dxsq=0.0d0
  116 do 124 m=1,3
       rji(m)=x(m,i)-x(m,j)
       djisq=djisq+rji(m)*rji(m)
       rjk(m)=x(m,k)-x(m,j)
       djksq=djksq+rjk(m)*rjk(m)
       rjl(m)=x(m,l)-x(m,j)
       djlsq=djlsq+rjl(m)*rjl(m)
       rixjx(m)=x(m,jx)-x(m,ix)
  124 dxsq=dxsq+rixjx(m)*rixjx(m)
  126 dji=sqrt(djisq)
      djk=sqrt(djksq)
      djl=sqrt(djlsq)
      dx=sqrt(dxsq)
  130 if(dx.lt.1.0d-12) dx=1.0d0
  132 do 136 m=1,3
       eji(m)=rji(m)/dji
       ejk(m)=rjk(m)/djk
  136 ejl(m)=rjl(m)/djl
  137 c1(1)=ejk(2)*ejl(3)-ejk(3)*ejl(2)
      c1(2)=ejk(3)*ejl(1)-ejk(1)*ejl(3)
      c1(3)=ejk(1)*ejl(2)-ejk(2)*ejl(1)
      c2(1)=ejl(2)*eji(3)-ejl(3)*eji(2)
      c2(2)=ejl(3)*eji(1)-ejl(1)*eji(3)
      c2(3)=ejl(1)*eji(2)-ejl(2)*eji(1)
      c3(1)=eji(2)*ejk(3)-eji(3)*ejk(2)
      c3(2)=eji(3)*ejk(1)-eji(1)*ejk(3)
  139 c3(3)=eji(1)*ejk(2)-eji(2)*ejk(1)
  140 det=eji(1)*c1(1)+eji(2)*c1(2)+eji(3)*c1(3)
      doti=0.0
  142 do 143 m=1,3
  143 doti=doti+ejk(m)*ejl(m)
  144 if(1.0d0-abs(doti))172,172,146
  146 sini=sqrt(1.0d0-doti*doti)
  147 sint=det/sini
  148 if(1.0d0-abs(sint))174,174,149
  149 cost=sqrt(1.0d0-sint*sint)
  150 tant=sint/cost
  155 do 168 m=1,3
  157  smi=((c1(m)/(cost*sini))-(tant*eji(m)))/dji
       nob=nob+1
       nr(nob)=noint
       nc(nob)=3*(i-1)+m
       b(nob)=dx*smi
  160  smk=((c2(m)/(cost*sini))-((tant*(ejk(m)-doti*ejl(m)))/(sini*sini)
     1))/djk
       nob=nob+1
       nr(nob)=noint
       nc(nob)=3*(k-1)+m
       b(nob)=dx*smk
  163  sml=((c3(m)/(cost*sini))-((tant*(ejl(m)-doti*ejk(m)))/(sini*sini)
     1))/djl
       nob=nob+1
       nr(nob)=noint
       nc(nob)=3*(l-1)+m
       b(nob)=dx*sml
  166  sum=smi+smk+sml
       nob=nob+1
       nr(nob)=noint
       nc(nob)=3*(j-1)+m
       b(nob)=-dx*sum
  168 continue
      go to 178
  170 joker=1
      go to 178
  172 joker=2
      go to 178
  174 joker=3
  178 return
      end
c
      subroutine tors (noat,noint,n1,n2,n3,n4,n5,n6,nob,joker,nbc)
**********************************************************************
c     this subroutine computes the b matrix elements for the torsion *
c     as defined by wilson.                                          *
c      i and l = the end atoms. i nearer observer.                   *
c      j and k = the central atoms. j nearer observer.               *
c*********************************************************************
      implicit real*8(a-h,o-z)
      common x(3,200),b(3500),nr(3500),nc(3500)
      dimension rij(3),rjk(3),rkl(3),rixjx(3),eij(3),ejk(3),ekl(3),
     1cr1(3),cr2(3)
  100 if(noat-n6)180,101,101
  101 if(noat-n5)180,102,102
  102 if(noat-n4)180,103,103
  103 if(noat-n3)180,104,104
  104 if(noat-n2)180,105,105
  105 if(noat-n1)180,106,106
  106 i=n1
      j=n2
      k=n3
      l=n4
      ix=n5
      jx=n6
      if(ix)110,110,112
  110 ix=1
      jx=1
  112 dijsq=0.0d0
      djksq=0.0d0
      dklsq=0.0d0
  115 dxsq=0.0d0
  116 do 124 m=1,3
       rij(m)=x(m,j)-x(m,i)
       dijsq=dijsq+rij(m)*rij(m)
       rjk(m)=x(m,k)-x(m,j)
       djksq=djksq+rjk(m)*rjk(m)
       rkl(m)=x(m,l)-x(m,k)
       dklsq=dklsq+rkl(m)*rkl(m)
       rixjx(m)=x(m,jx)-x(m,ix)
  124 dxsq=dxsq+rixjx(m)*rixjx(m)
  126 dij=sqrt(dijsq)
      djk=sqrt(djksq)
      dkl=sqrt(dklsq)
      dx=sqrt(dxsq)
  130 if(dx.lt.1.0d-12) dx=1.0
  132 do 136 m=1,3
       eij(m)=rij(m)/dij
       ejk(m)=rjk(m)/djk
  136 ekl(m)=rkl(m)/dkl
  138 cr1(1)=eij(2)*ejk(3)-eij(3)*ejk(2)
      cr1(2)=eij(3)*ejk(1)-eij(1)*ejk(3)
      cr1(3)=eij(1)*ejk(2)-eij(2)*ejk(1)
      cr2(1)=ejk(2)*ekl(3)-ejk(3)*ekl(2)
      cr2(2)=ejk(3)*ekl(1)-ejk(1)*ekl(3)
  142 cr2(3)=ejk(1)*ekl(2)-ejk(2)*ekl(1)
  143 dotpj=0.0d0
      dotpk=0.0d0
  145 do 147 m=1,3
      dotpj=dotpj-eij(m)*ejk(m)
  147 dotpk=dotpk-ejk(m)*ekl(m)
  148 if(1.0d0-abs(dotpj))182,182,149
  149 if(1.0d0-abs(dotpk))182,182,150
  150 sinpj=sqrt(1.0d0-dotpj*dotpj)
      sinpk=sqrt(1.0d0-dotpk*dotpk)
  152 do 164 m=1,3
       smi=-cr1(m)/(dij*sinpj*sinpj)
       nob=nob+1
       nr(nob)=noint
       nc(nob)=3*(i-1)+m
       b(nob)=dx*smi
       f1=(cr1(m)*(djk-dij*dotpj))/(djk*dij*sinpj*sinpj)
       f2=(dotpk*cr2(m))/(djk*sinpk*sinpk)
       smj=f1-f2
       nob=nob+1
       nr(nob)=noint
       nc(nob)=3*(j-1)+m
       b(nob)=dx*smj
       sml= cr2(m)/(dkl*sinpk*sinpk)
       nob=nob+1
       nr(nob)=noint
       nc(nob)=3*(l-1)+m
       b(nob)=dx*sml
       sum=smi+smj+sml
       nob=nob+1
       nr(nob)=noint
       nc(nob)=3*(k-1)+m
       b(nob)=-dx*sum
  164 continue
      go to 186
  180 joker=1
      go to 186
  182 joker=2
  186 return
      end
c
      subroutine libe (noat,noint,n1,n2,n3,n4,n5,n6,nob,joker,no2)
c*********************************************************************
c     this subroutine computes the b matrix elements for a linear    *
c     bend or for a pair of perpendicular linear bends.              *
c      n1=no2 the number of the second coordinate.                   *
c      i and k = the end atoms.                                      *
c      j= the central atom.                                          *
c     a gives the cartesian coordinates of a point in space, such    *
c      that the vector from atom j to point a is perpendicular to    *
c      the line i-j-k and serves to orient the coordinates in space. *
c*********************************************************************
      implicit real*8(a-h,o-z)
      common x(3,200),b(3500),nr(3500),nc(3500)
      dimension a(3),rji(3),rjk(3),rixjx(3),un(3),unit(3),up(3),eji(3),
     1ejk(3)
      jstop=0
  100 if(noat-n6)160,101,101
  101 if(noat-n5)160,102,102
  102 if(noat-n4)160,103,103
  103 if(noat-n3)160,104,104
  104 if(noat-n2)160,105,105
  105 read(7,24)(a(i),i=1,3)
   24 format(3f12.6)
      if(n4)160,109,108
  108 i=n2
      j=n3
      k=n4
      jstop=1
      go to 110
  109 i=n1
      j=n2
      k=n3
  110 ix=n5
      jx=n6
      if(ix)111,111,112
  111 ix=1
      jx=1
  112 djisq=0.0d0
      djksq=0.0d0
      dxsq=0.0d0
  116 dajsq=0.0d0
  117 do 124 m=1,3
       rji(m)=x(m,i)-x(m,j)
       djisq=djisq+rji(m)*rji(m)
       rjk(m)=x(m,k)-x(m,j)
       djksq=djksq+rjk(m)*rjk(m)
       rixjx(m)=x(m,jx)-x(m,ix)
       dxsq=dxsq+rixjx(m)*rixjx(m)
       un(m)=a(m)-x(m,j)
  124 dajsq=dajsq+un(m)*un(m)
  126 dji=sqrt(djisq)
      djk=sqrt(djksq)
      dx=sqrt(dxsq)
      daj=sqrt(dajsq)
  130 if(dx.lt.1.0d-12) dx=1.0d0
  132 dotj=0.0d0
      dotp=0.0d0
  134 do 140 m=1,3
       eji(m)=rji(m)/dji
       ejk(m)=rjk(m)/djk
       unit(m)=un(m)/daj
       dotj=dotj+eji(m)*ejk(m)
  140 dotp=dotp+eji(m)*unit(m)
      test=(abs(dotj)-1.0d0)
      if(0.0001-abs(test))162,142,142
  142 if(0.00005-abs(dotp))162,143,143
  143 up(1)=ejk(2)*unit(3)-ejk(3)*unit(2)
      up(2)=ejk(3)*unit(1)-ejk(1)*unit(3)
      up(3)=ejk(1)*unit(2)-ejk(2)*unit(1)
  146 do 149 m=1,3
       nob=nob+1
       nr(nob)=noint
       nc(nob)=3*(i-1)+m
       b(nob)=-dx*unit(m)/ dji
       nob=nob+1
       nr(nob)=noint
       nc(nob)=3*(k-1)+m
       b(nob)=-dx*unit(m)/ djk
       nob=nob+1
       nr(nob)=noint
       nc(nob)=3*(j-1)+m
       b(nob)=dx*(1.0d0/dji+1.0d0/djk)*unit(m)
  149 continue
      if(jstop)164,164,150
  150 do 158 m=1,3
       nob=nob+1
       nr(nob)=no2
       nc(nob)=3*(i-1)+m
       b(nob)=-dx*up(m)/dji
       nob=nob+1
       nr(nob)=no2
       nc(nob)=3*(k-1)+m
       b(nob)=-dx*up(m)/djk
       nob=nob+1
       nr(nob)=no2
       nc(nob)=3*(j-1)+m
       b(nob)=dx*(1.0d0/dji+1.0d0/djk)*up(m)
  158 continue
      go to 164
  160 joker=1
      go to 164
  162 joker=2
  164 return
      end
