C**********************************************************************
C                 Copyright 1992-1996, Robert Fraczkiewicz            *
C                           All rights reserved.                      *
C**********************************************************************
C This program is free software; you can redistribute it and/or modify*
C it under the terms of the GNU General Public License as published by*
C the Free Software Foundation; either version 1, or (at your option) *
C any later version.                                                  *
C                                                                     *
C This program is distributed in the hope that it will be useful,     *
C but WITHOUT ANY WARRANTY; without even the implied warranty of      *
C MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU    *
C General Public License for more details.                            *
C                                                                     *
C You should  have received a copy of the GNU General Public License  *
C along with this program; if not, write to the Free Software         *
C Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.           *
c**********************************************************************
c                                                                     *
c             Program OV by Robert Fraczkiewicz, Jun 22, 95           *
c Calculates overtones and combination bands from experimental data   *
c                         Version 1.0                                 *
c                                                                     *
c**********************************************************************
c                                                                     *
c     dimensions:                                                     *
c     fcy,int,pol                    - NFCY                           *
c     sum,nn,n1,n2                   - NFCY*(NFCY+1)/2                *
c                                                                     *
c**********************************************************************
c                                                                     *
c  Format of the input file:                                          *
c                                                                     *
c  fcy,int,pol                                                        *
c  (f6.1,1x,a5,1x,a5)                                                 *
c                                                                     *
c  where:                                                             *
c                                                                     *
c        fcy: observed wavenumbers                                    *
c        int: intensity label (vw,w,m,s,vs;sh)                        *
c        pol: polarization label (p,dp,ap)                            *
c                                                                     *
c**********************************************************************

      program ov
      integer nfcy,nsum
      real tol_left,tol_right
      parameter(nfcy=300,nsum=nfcy*(nfcy+1)/2,tol_left=2.0,
     .          tol_right=5.0)
      character*20 name,u
      character*5 int(nfcy),pol(nfcy),pom
      real*4 fcy(nfcy),sum(nsum),limit
      integer nn(nsum),n1(nsum),n2(nsum)
      logical change
c**********************************************************************
      ve=.false.
      if(iargc().ne.2) then
       print *,'^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^'
       print *
       print *,'                         O V C'
       print *
       print *,'                     Version 1.0'
       print *
       print *,'               Overtones and combinations'
       print *
       print *,'______________________________________________________'
       print *
       print *,'USAGE:'
       print *
       print *,'     ovc filename limit'
       print *
       print *,'  Results of the program will be stored in the output'
       print *,'  file with extension ".ovout".'
       print *,'  OVC requires an input file with extension ".ovin"'
       print *
       print *,'PARAMETERS:'
       print *
       print *,'  Length of the filename cannot exceed 20 characters.'
       print *,'  The ".ovin" extension will be automatically added at'
       print *,'  the end of filename.'
       print *
       print *,'  limit - highest expected wavenumber of the'
       print *,'          fundamental vibrations'
       print *
       print *,'______________________________________________________'
       stop 1
      endif
      call getarg(1,name)
      l_name=namlen(name)
      call getarg(2,u)
      read(u,*) limit
c*******************Read input*****************************************
      open(unit=8, file=name(1:l_name)//'.ovin', status='old')
      n=0
   10 n=n+1
      read(8,900,err=9000,end=20) fcy(n),int(n),pol(n)
      goto 10
   20 n=n-1
      close(unit=8)
c******************Sort the input data*********************************
   30 change=.false.
      do i=2,n
       if(fcy(i).lt.fcy(i-1)) then
        change=.true.
        fc=fcy(i)
        fcy(i)=fcy(i-1)
        fcy(i-1)=fc
        pom=int(i)
        int(i)=int(i-1)
        int(i-1)=pom
        pom=pol(i)
        pol(i)=pol(i-1)
        pol(i-1)=pom
       endif
      enddo
      if(change) goto 30
      nrow=0
      do j=1,n
       nrow=nrow+j-1
       do i=1,j
        sum(nrow+i)=fcy(i)+fcy(j)
       enddo
      enddo
      m=0
      do k=1,n
       nrow=0
       j=0
   40  j=j+1
       nrow=nrow+j
       if(sum(nrow).le.(fcy(k)-tol_left)) goto 40
       nrowj=nrow-j
       if(sum(nrowj+1).le.(fcy(k)+tol_right)) then
        i=0
   50   i=i+1
        if(i.gt.j) goto 40
        if(sum(nrowj+i).le.(fcy(k)-tol_left)) goto 50
        if(sum(nrowj+i).ge.(fcy(k)+tol_right)) goto 40
        if(fcy(j).le.limit.and.fcy(i).lt.limit) then
         m=m+1
         if(m.gt.nsum) print *,'Subscript out of range, m = ',m
         nn(m)=k
         n1(m)=j
         n2(m)=i
        endif
        goto 50
       endif
      enddo
      open(unit=9, file=name(1:l_name)//'.ovout', status='unknown')
      do i=1,m
       write(9,910) fcy(nn(i)),int(nn(i)),pol(nn(i)),
     .              fcy(n1(i)),int(n1(i)),pol(n1(i)),
     .              fcy(n2(i)),int(n2(i)),pol(n2(i)),
     .              fcy(n1(i))+fcy(n2(i))
      enddo
      close(unit=9)
      stop 0
  900 format(f6.1,1x,a5,1x,a5)
  910 format(1x,'[',f6.1,2(1x,a5),'] <=> [',f6.1,2(1x,a5),
     .       '] + [',f6.1,2(1x,a5),'] = ',f6.1)
c++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 9000 print *,'Required format of the .ovin file:'
      print *
      print *,'fcy,int,pol   '
      print *,'(f6.1,1x,a5,1x,a5)'
      print *
      print *,'where:'
      print *
      print *,'       fcy: observed wavenumbers'
      print *,'       int: intensity label (vw,w,m,s,vs;sh)'
      print *,'       pol: polarization label (p,dp,ap)'
      stop 2
      end

      integer function namlen(stri)
      character*20 stri
      do i=1,len(stri)
       if(stri(i:i).eq.' ') goto 2
      enddo
    2 namlen=i-1
      return
      end
