C**********************************************************************
C                 Copyright 1992-1996, Robert Fraczkiewicz            *
C                           All rights reserved.                      *
C**********************************************************************
C This program is free software; you can redistribute it and/or modify*
C it under the terms of the GNU General Public License as published by*
C the Free Software Foundation; either version 1, or (at your option) *
C any later version.                                                  *
C                                                                     *
C This program is distributed in the hope that it will be useful,     *
C but WITHOUT ANY WARRANTY; without even the implied warranty of      *
C MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU    *
C General Public License for more details.                            *
C                                                                     *
C You should  have received a copy of the GNU General Public License  *
C along with this program; if not, write to the Free Software         *
C Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.           *
c**********************************************************************
c                                                                     *
c       Program "Normal Mode Sorting" by Robert Fraczkiewicz          *
c                     Jun 03, 95,  Version 1.7                        *
c                                                                     *
c**********************************************************************
c Program matches normal modes of isotopic molecules by distance      *
c calculation between corresponding PED vectors. PED data are read    *
c from binary file ".ped" generated by program FR when option -1 is   *
c used. The ".rein" file before running FR should be ordered as       *
c follows:                                                            *
c                                                                     *
c  symmetry block 1                                                   *
c   molecule    1                                                     *
c   molecule    2                                                     *
c   ..........                                                        *
c   molecule nmol                                                     *
c  symmetry block 2                                                   *
c   molecule    1                                                     *
c   molecule    2                                                     *
c   ..........                                                        *
c   molecule nmol                                                     *
c  ...............                                                    *
c  symmetry block nsym                                                *
c   molecule    1                                                     *
c   molecule    2                                                     *
c   ..........                                                        *
c   molecule nmol                                                     *
c                                                                     *
c PED sorting will be related to molecule 1. The method works well in *
c the case of small isotopic shifts, but usually fails when the PED   *
c changes upon isotopic substitution become significant. In such cases*
c the program points out to several most probable matches.            *
c                                                                     *
c**********************************************************************
c                                                                     *
c     INPUT FILES:                                                    *
c                                                                     *
c     ".ped" - binary file containing PED vectors generated by FR     *
c              with option -1                                         *
c                                                                     *
c**********************************************************************
c                                                                     *
c     OUTPUT FILES:                                                   *
c                                                                     *
c     ".tab" - table of observed and calculated frequencies, list of  *
c              unmatched modes and average deviation.                 *
c     ".pdi" - table of PED distances ("tag" indicates the number of  *
c              succesful matches).                                    *
c     ".sft" - table of observed and calculated isotopic shifts.      *
c                                                                     *
c**********************************************************************
c                                                                     *
c     DIMENSIONS:                                                     *
c                                                                     *
c     fq,fqc,idist,distmin,fq1,fqc1,ntag  - NDIM                      *
c     rr1,rr2                             - NDIMxNINT*(NINT+1)/2      *
c     out                                 - NDIMxNDIM                 *
c     comment                             - NBL                       *
c     shift                               - NDIMxNBL                  *
c                                                                     *
c**********************************************************************

      program pedsort
      implicit real*8(a-h,o-z)
      include 'Sizes'
      integer nped
      parameter(nped=6105)
      character*20 name
      character*80 comment1,comment(nbl),u
      dimension rr1(ndim,nped),rr2(ndim,nped),fq(ndim),fq1(ndim),
     .fqc(ndim),fqc1(ndim),idist(ndim),distmin(ndim),ntag(ndim),
     .out(ndim,ndim),shift(ndim,nbl)
      logical ve
c**********************************************************************
      ve=.false.
      if(iargc().ne.1.and.iargc().ne.3) then
       print *,'^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^'
       print *
       print *,'                    P E D S O R T'
       print *
       print *,'                     Version 1.7'
       print *
       print *,'                  Normal Mode Sorting'
       print *
       print *,'______________________________________________________'
       print *
       print *,'SYNOPSIS:'
       print *
       print *,'     pedsort -h'
       print *
       print *,'      or'
       print *
       print *,'     pedsort filename nb nsym'
       print *
       print *,'PARAMETERS: '
       print *
       print *,'      filename - core name of the input and output'
       print *,'                 files. Its length cannot exceed 14'
       print *,'                 characters. Appropriate extensions'
       print *,'                 are added by default.'
       print *
       print *,'      nb - total number of blocks defined in the '
       print *,'           .rein file'
       print *
       print *,'      nsym - number of different symmetry blocks'
       print *
       print *,'DESCRIPTION:'
       print *
       print *,'      Program matches normal modes of isotopic '
       print *,'      molecules by distance calculation between'
       print *,'      corresponding PED vectors. PED data are read'
       print *,'      from binary file ".ped" generated by program FR'
       print *,'      when option -1 is used. '
       print *
       print *,'OPTIONS:'
       print *
       print *,'  -h  Program prints required format of .rein file'
       print *
       print *,'______________________________________________________'
       stop 1
      endif
      call getarg(1,name)
      if(name.eq.'-h') go to 9000
      l_name=namlen(name)
      call getarg(2,u)
      read(u,*) nb
      call getarg(3,u)
      read(u,*) nsym
c**********************************************************************
      open(unit=8, file=name(1:l_name)//'.pdi', status='unknown')
      open(unit=9, file=name(1:l_name)//'.sft', status='unknown')
      open(unit=11,file=name(1:l_name)//'.tab', status='unknown')
      open(unit=10,file=name(1:l_name)//'.ped', status='old',
     .     form='unformatted', disp='delete')
      if(float(nb/nsym).ne.int(nb/nsym)) then
       print *,'Wrong number of symmetry blocks !'
       stop 5
      endif
      nmol=nb/nsym
      nexp=0
      devi=0.d0
c*****************Read PED for the first molecule*********************
      do n=1,nsym
       read(10) nz1,nq1,comment1
       write(8,*)
       write(8,'(1x,a79)') comment1
       do l=1,nq1
        read(10) fq1(l),fqc1(l)
        out(l,1)=fq1(l)
        out(l,2)=fqc1(l)
        if(fq1(l).ne.0.0) then
         out(l,3)=fqc1(l)-fq1(l)
         nexp=nexp+1
         devi=devi+abs(out(l,3))
        else
         out(l,3)=0.0
        endif
        read(10) (rr1(l,j), j=1,nz1)
        write(8,20) l,fq1(l),fqc1(l)
       enddo
       i3=0
       write(11,'(1x,a79)') comment1
c***************Read data for isotopic molecules********************
       do i=2,nmol
        i3=i3+3
        read(10) nz2,nq2,comment(i)
        if(nz1.ne.nz2.or.nq1.ne.nq2) then
         print *,'Error in the input data !'
         print *,'Symmetry block ',n
         print *,'Molecule       ',i
         print *,'Block dimensions nq1,nq2 =            ',nq1,nq2
         print *,'Requested Z-matrix elements nz1,nz2 = ',nz1,nz2
         print *,'Please, check the format of your .rein file'
         print *,'and run FR again.'
         print *,'The .rein file  should be ordered as follows:'
         print *
         go to 9000
        endif
        do l=1,nq2
         ntag(l)=0
        enddo
        do l=1,nq2
         read(10) fq(l),fqc(l)
         read(10) (rr2(l,j), j=1,nz2)
        enddo
        do l1=1,nq1
         distmin(l1)=1.d38
         do l2=1,nq2
          dist=0.d0
          do j=1,nz1
           pom=rr1(l1,j)-rr2(l2,j)
           dist=dist+pom*pom
          enddo
          if(dist.lt.distmin(l1)) then
           distmin(l1)=dist
           lmin=l2
          endif
         enddo
         ntag(lmin)=ntag(lmin)+1
         idist(l1)=lmin
        enddo
        write(8,*)
        write(8,'(1x,a79)') comment(i)
        product1=1.d0
        product2=1.d0
        do l=1,nq2
         out(l,i3+1)=fq(idist(l))
         out(l,i3+2)=fqc(idist(l))
         if(fq(idist(l)).ne.0.0) then
          out(l,i3+3)=fqc(idist(l))-fq(idist(l))
          nexp=nexp+1
          devi=devi+abs(out(l,i3+3))
         else
          out(l,i3+3)=0.0
         endif
         shift(l,i)=fqc(idist(l))-fqc1(l)
         write(8,30) l,fq(idist(l)),fqc(idist(l)),distmin(l),
     .               ntag(idist(l))
        enddo
        do l=1,nq2
         if(ntag(l).eq.0) then
          write(8,31) comment(i),fq(l),fqc(l)
          write(9,31) comment(i),fq(l),fqc(l)
          write(11,31) comment(i),fq(l),fqc(l)
         endif
        enddo
        write(11,'(1x,a79)') comment(i)
       enddo
       write(9,*)
       write(9,55) comment1,(comment(i),i=2,nmol)
       do l=1,nq1
        write(9,40) l,fq1(l),fqc1(l),(shift(l,i),i=2,nmol)
       enddo
       do l=1,nq1
        write(11,60) (out(l,lll), lll=1,i3+3)
       enddo
      enddo
      write(11,*) 'Number of observed frequencies : ',nexp
      write(11,*) 'Average deviation : ',devi/float(nexp)
      close(unit=8)
      close(unit=9)
      close(unit=10)
      close(unit=11)
   20 format(1x,'f-cy #',i3,'   obs. ',f8.2,'   calc. ',f8.2) 
   30 format(1x,'f-cy #',i3,'   obs. ',f8.2,'   calc. ',f8.2,
     .'   dist. ',e9.2,'   tag ',i3)
   31 format(1x,a48,': no match for :   obs. ',f8.2,'   calc. ',f8.2)
   40 format(i3,2(1x,f8.2),20(1x,'|',1x,f8.2,9x))
   50 format(1x,'|',5(1x,f8.2))
   55 format(2x,'#',20(a19,'|'))
   60 format(60(1x,f8.2))
   70 format(1x,'Products:',3x,e11.4,7x,'|',
     .       7x,e11.4/1x,'Product ratio: ',f12.9)
      stop 0
c++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
9000  print *,'Required format of the .rein file:'
      print *
      print *,'  symmetry block 1 '
      print *,'   molecule 1      '
      print *,'   molecule 2      '
      print *,'   ..........      '
      print *,'   molecule nmol   '
      print *,'  symmetry block 2 '
      print *,'   molecule 1      '
      print *,'   molecule 2      '
      print *,'   ..........      '
      print *,'   molecule nmol   '
      print *,'  ...............  '
      print *,'  symmetry block nsym '
      print *,'   molecule 1      '
      print *,'   molecule 2      '
      print *,'   ..........      '
      print *,'   molecule nmol   '
      print *
      print *,'PED sorting will be related to molecule 1 '
      stop 2
      end

      integer function namlen(stri)
      implicit real*8(a-h,o-z)
      character*20 stri
      do i=1,len(stri)
       if(stri(i:i).eq.' ') goto 2
      enddo
    2 namlen=i-1
      return
      end
