C**********************************************************************
C                 Copyright 1992-1996, Robert Fraczkiewicz            *
C                           All rights reserved.                      *
C**********************************************************************
C This program is free software; you can redistribute it and/or modify*
C it under the terms of the GNU General Public License as published by*
C the Free Software Foundation; either version 1, or (at your option) *
C any later version.                                                  *
C                                                                     *
C This program is distributed in the hope that it will be useful,     *
C but WITHOUT ANY WARRANTY; without even the implied warranty of      *
C MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU    *
C General Public License for more details.                            *
C                                                                     *
C You should  have received a copy of the GNU General Public License  *
C along with this program; if not, write to the Free Software         *
C Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.           *
c**********************************************************************
c                                                                     *
c     Program "Force Field Refinement" by Robert Fraczkiewicz         *
c                      Mar 96, version 6.6                            *
c                                                                     *
c**********************************************************************
c The program optimizes harmonic force field based on a novel method  *
c of modified regularized least-squares developed by author. Due to   *
c regularization the method is resistant to the Jacobian matrix       *
c singularities. This iterative convergence usually requires compro-  *
c mising the goodness-of-fit because of so-called regularization      *
c error (r.e., bias). Unlike other similar procedures, this program   *
c manages to minimize the r.e. in several quasi-Newton iterative      *
c steps. The regularized least-squares iterations are performed by    *
c a truncated singular value decomposition (TCSVD) of the Jacobian    *
c matrix. Quasi-Newton minimization of r.e. is done with the aid of   *
c Broyden-Fletcher-Goldfarb-Shanno (BFGS) procedure on specially      *
c prepared penalty function. The TSVD threshold level (rlevel) and    *
c penalty parameter (efactor) must be specified by a user in the      *
c command line. Please, refer to literature for further details.      *
c**********************************************************************
c                                                                     *
c     INPUT FILES:                                                    *
c                                                                     *
c     ".rein" - main input file containing force field and obseved    *
c               vibrational frequencies                               *
c     ".zmat" - Z-matrix (output of ZSYM4)                            *
c     ".gmat" - G-matrix (output of GMAT4)                            *
c                                                                     *
c**********************************************************************
c                                                                     *
c     OUTPUT FILES:                                                   *
c                                                                     *
c     ".fi" - refined force field                                     *
c     "fort.7" - log file. Contains: goodness-of-fit, fitting         *
c                potential, initial Fourier coefficients, intermediate*
c                force fields and direction of change, and correlation*
c                matrices.                                            *
c     "fort.8" - correlation matrix of the full (i.e., non-truncated) *
c                initial force constant set.                          *
c                                                                     *
c**********************************************************************
c                                                                     *
c     BLAS AND LAPACK ROUTINES:                                       *
c                                                                     *
c     dsymm - symmetric matrix product                                *
c     dgemm - general matrix product                                  *
c     dgemv - general matrix-vector product                           *
c     dspev - eigenanalysis of real symmetric matrix in packed storage*
c     dgesvd - singular value decomposition of a real general marix   *
c     dsyev - eigenanalysis of a real symmetric matrix                *         c     dsyrk - symmetric matrix update                                 *
c                                                                     *
c On SGI machines LAPACK library is specified by f77 compiler option  *
c -lcomplib.sgimath                                                   *
c                                                                     *
c**********************************************************************
c                                                                     *
c     DIMENSIONS:                                                     *
c                                                                     *
c     work                           - LWORK                          *
c     nfo,nr,nc,z                    - NBLxNDIMZ                      *
c     numb                           - NBLxNFC                        *
c     q                              - NDIMxNDIMxNBL                  *
c     h0                             - [NDIM*(NDIM+1)/2]xNBL          *
c     nq,nz,comment                  - NBL                            *
c     fq,nzfq,sfq                    - NBLxNDIM                       *
c     fi,a,fi0,sigma,gamma,hgamma,ncc- NFC                            *
c     g,w,opt                        - NFCxNFC                        *
c     b                              - NBL*NDIM                       *
c     aa,u                           - NBL*NDIMxNFC                   *
c                                                                     *
c**********************************************************************

      program ref
      implicit real*8(a-h,o-z)
      include 'Sizes'
      integer inp,maxiter
      character*4 optvector
      parameter(optvector='vrei',inp=1)
      character*20 name,cr
      character*80 comment(nbl),dummy
      logical change,tmpin,ve
      dimension g(nfc,nfc),w(nfc,nfc),opt(nfc,nfc),a(nfc),work(lwork),
     .          sigma(nfc),gamma(nfc),hgamma(nfc)
      equivalence (sigma(1),work(1)),(gamma(1),work(nfc+1)),
     .            (hgamma(1),work(2*nfc+1))
      real tarray(2)
      common /jacob/ aa(jarow,nfc),u(jarow,nfc),ncc(nfc)
     . /qmat/ q(ndim,ndim,nbl),fq(nbl,ndim),sfq(nbl,ndim),
     .        nq(nbl),nz(nbl),nzfq(nbl,ndim),nfo(nbl,ndimz)
     . /hmat/ h0(nkdim,nbl),b(jarow),fi(nfc),fi0(nfc)
     . /hkmatrices/ nr(nbl,ndimz),nc(nbl,ndimz),z(nbl,ndimz),
     . numb(nbl,nfc)
c**********************************************************************
      name=''
      ve=.false.
      rlevel=0.1
      eee=1.d6
      maxiter=1000
      if(iargc().lt.1) then 
       print *
       print *,' /--------------------------------------------------\ '
       print *,'|           RRRRRRRR   EEEEEEEEE  FFFFFFFFF          |'
       print *,'|           R       R  E          F                  |'
       print *,'|           R       R  E          F                  |'
       print *,'|           RRRRRRRR   EEEEEEE    FFFFFFF            |'
       print *,'|           R   R      E          F                  |'
       print *,'|           R    R     E          F                  |'
       print *,'|           R      R   EEEEEEEEE  F                  |'
       print *,'|                                                    |'
       print *,'|                     version 6.6                    |'
       print *,'|                                                    |'
       print *,'|            Force Field Refinement Program          |'
       print *,'|                                                    |'
       print *,'|                                                    |'
       print *,'|                           by Robert Fraczkiewicz   |'
       print *,'|                      University of Houston, 1992-5 |'
       print *,' \--------------------------------------------------/ '
       print *
       print *,'______________________________________________________'
       print *
       print *,'SYNOPSIS: '
       print *
       print *,'      ref6 [options] filename'
       print *
       print *,'PARAMETERS: '
       print *
       print *,'      filename - core name of the input and output'
       print *,'                 files. Its length cannot exceed 14'
       print *,'                 characters. Appropriate extensions'
       print *,'                 are added by default.'
       print *
       print *,'DESCRIPTION:'
       print *
       print *,' The program optimizes harmonic force field based on'
       print *,' a novel method of modified regularized least-squares'
       print *,' developed by author. Due to regularization the method'
       print *,' is resistant to the Jacobian matrix singularities.'
       print *,' This iterative convergence usually requires compro-'
       print *,' mising the goodness-of-fit because of so-called'
       print *,' regularization error (r.e., bias). Unlike other '
       print *,' similar procedures, this program manages to minimize'
       print *,' the r.e. in several quasi-Newton iterative steps.'
       print *,' The regularized least-squares iterations are '
       print *,' performed by a truncated singular value decomposition'
       print *,' (TCSVD) of the Jacobian matrix. Quasi-Newton'
       print *,' minimization of r.e. is done with the aid of'
       print *,' Broyden-Fletcher-Goldfarb-Shanno (BFGS) procedure'
       print *,' on specially prepared penalty function. The TSVD'
       print *,' threshold level (rlevel) and penalty parameter'
       print *,' (efactor) must be specified by a user in the'
       print *,' command line. Please, refer to literature for further' 
       print *,' details.'
       print *,' The correlation matrices stored in files fort.7 and'
       print *,' allow to spot nearly-dependent force constants.'
       print *
       print *,'OPTIONS:'
       print *
       print *,'  -v  verbose flag. Program will produce more output'
       print *,'      to the screen.'
       print *
       print *,'  -r <rlevel> - truncation parameter for filtering'
       print *,'                the TSVD expansion. Defaults to 0.1.'
       print *
       print *,'  -e <efactor> - the penalty parameter (ro) used in'
       print *,'                quasi-Newton optimization of Picard'
       print *,'                functions. Defaults to 1.0E6.'
       print *
       print *,'  -i <maxiter> - maximum allowable number of'
       print *,'                iterations. Defaults to 1000.'
       print *
       print *,'OUTPUT:'
       print *
       print *,'  Results of the program will be stored in the '
       print *,'  following output files:'
       print *
       print *,'   - "*.fi" contains final force field, if the'
       print *,'     program terminated normally'
       print *
       print *,'   - "fort.8": temporary file containing initial'
       print *,'     correlation matrix of force constants (i.e.,'
       print *,'     before removing the coordinate redundancies)'
       print *
       print *,'   - "fort.7": the log file containing detailed'
       print *,'     information about every refinement iteration:'
       print *,'     weighted square deviation, refinable and '
       print *,'     unrefinable components of the b-vector,'
       print *,'     normalized vector u''b (helpful in determining'
       print *,'     the right value of the "rlevel" parameter),'
       print *,'     and current values of force constants, their'
       print *,'     increments and (non-redundant) correlation matrix'
       print *,'     before each quasi-Newton optimization.'
       print *,'     When option -v is used, the current force field'
       print *,'     is saved in fort.7 after every iteration.'
       print *,'     UNIX allows for viewing the file during the'
       print *,'     program exacution.'
       print *
       print *,'______________________________________________________'
       stop 7
      endif
      i=1
   50 call getarg(i,dummy)
      if(dummy(1:1).eq.'-') then
       m=inopt(dummy(2:2),optvector)
       goto (1,2,3,4) m
       print *,'Invalid option: '//dummy(1:2)
       stop 7
      else
       name=dummy
       l_name=namlen(name)
       goto 60
      endif
    1 ve=.true.
      goto 60
    2 i=i+1
      call getarg(i,dummy)
      read(dummy,*,err=71) rlevel
      goto 60
    3 i=i+1
      call getarg(i,dummy)
      read(dummy,*,err=71) eee
      goto 60
    4 i=i+1
      call getarg(i,dummy)
      read(dummy,*,err=71) maxiter
      goto 60
   71 print *,' = Error reading parameter line ='
      print *,' = Are all the options and parameters correct ? ='
      stop 7
   60 i=i+1
      if(i.le.iargc()) goto 50
      if(name.eq.'') then
       print *,' = Hey, where''s the filename ? ='
       stop 7
      endif
      if(ve) then
       print *,i-1,' arguments detected'
       print *,'Name    : ',name(1:l_name)
       print *,'Rlevel  : ',rlevel
       print *,'Efactor : ',eee
       print *,'Maxiter : ',maxiter
       print *
      endif
c**************Read initial set of force constants*****************
      print *,'Reading '//name(1:l_name)//'.rein'
      open(unit=9,file=name(1:l_name)//'.rein',err=9990,status='old')
      read(9,*) nf
      read(9,*) (fi(i),i=1,nf)
      do j=1,nf
       fi0(j)=fi(j)
      enddo
      read(9,*) nvar
      if(nvar.lt.0) then
       nvar=nf+nvar
       nfixed=nf-nvar
       read(9,*) (nr(1,i),i=1,nfixed)
       l=1
       j=1
       i=0
       do while(l.le.nfixed.or.j.le.nvar)
        i=i+1
        if(i.eq.nr(1,l)) then
         l=l+1
        else
         ncc(j)=i
         j=j+1
        endif
       enddo
      else
       nfixed=nf-nvar
       if(nfixed.eq.0) then
        do i=1,nvar
         ncc(i)=i
        enddo
       else
        read(9,*) (ncc(i), i=1,nvar)
       endif
      endif 
      read(9,*) nb
      nroo=0
      rewind inp
      do i=1,nb
c**************Read experimental frequencies*******************
c nblok - number of symmetry block, nmol - number of molecule *
c**************************************************************
       read(9,'(i2,1x,i2,1x,a80)') nblok,nmol,comment(i)
       read(9,*) nfq,nq(i)
       read(9,*) (fq(i,j), j=1,nfq)
       kk=0
       do j=1,nfq
        if(fq(i,j).eq.0.0) then 
         kk=kk+1
         nzfq(i,kk)=j
        endif
        fq(i,j)=factor*fq(i,j)*fq(i,j)
        sfq(i,j)=sqrt(fq(i,j))
       enddo
c**************Read and diagonalize G-matrix block*************
       call getname(name,l_name,nblok,nmol,cr,l_cr)
       call readg(cr,l_cr,i,nfq,w,nfc,nq(i),npom,ve)
       write(inp) i,nq(i),npom,((w(k,l), l=1,npom), k=1,nq(i))
c**************Read a Z matrix block**********************************
c*Z matrix must be ordered in the descending order of force constants*
c*********************************************************************
       call getname(name,l_name,nblok,0,cr,l_cr)
       print *,'Reading '//cr(1:l_cr)//'.zmat'
       open(unit=7,file=cr(1:l_cr)//'.zmat',status='old',form=
     . 'unformatted')
       read(7) noz
       read(7)(nr(i,l),nc(i,l),nfo(i,l),z(i,l),l=1,noz)
       close(unit=7)
c*********************Prepare Zk and H0 matrices*******************
       do k=1,nq(i)
        do l=1,nq(i)
         g(k,l)=0.0
        enddo
       enddo
       if(nfixed) 75,75,70
   70  change=.false.
       do l=1,noz-1
        ll=l+1
        tmpin=in(nfo(i,l),ncc,nvar,nfc)
        if(.not.tmpin) then
         tmpin=in(nfo(i,ll),ncc,nvar,nfc)
         if(tmpin) then
          change=.true.
          n1=nr(i,l)
          n2=nc(i,l)
          n3=nfo(i,l)
          zz=z(i,l)
          nr(i,l)=nr(i,ll)
          nc(i,l)=nc(i,ll)
          nfo(i,l)=nfo(i,ll)
          z(i,l)=z(i,ll)
          nr(i,ll)=n1
          nc(i,ll)=n2
          nfo(i,ll)=n3
          z(i,ll)=zz
         endif
	endif
       enddo
       if(change) goto 70 
       k=0
   80  k=k+1        
       if(k.le.noz) then
        tmpin=in(nfo(i,k),ncc,nvar,nfc)
        if(tmpin) goto 80
       endif
       nzz=k-1
       do j=nzz+1,noz
        nrow=nr(i,j)
        ncol=nc(i,j)
        g(nrow,ncol)=g(nrow,ncol)+z(i,j)*fi(nfo(i,j))
        g(ncol,nrow)=g(nrow,ncol)
       enddo
       call dsymm('L','U',nq(i),npom,1.0d0,g,nfc,w,nfc,0.d0,opt,nfc)
       call dgemm('T','N',npom,npom,nq(i),1.0d0,w,nfc,opt,nfc,0.d0,
     .             g,nfc)
       np=0
       do l=1,npom
        do k=1,l
          h0(np+k,i)=g(k,l)
        enddo
        np=np+l
       enddo
       go to 77
   75  nzz=noz
       np=0
       do l=1,npom
        do k=1,l
          h0(np+k,i)=0.d0
        enddo
        np=np+l
       enddo
c**************Order active force constants********************
   77  do k=1,nvar
        numb(i,k)=0.d0
       enddo
       k=nfo(i,1)
       nz(i)=0
       do l=1,nzz
        if(nfo(i,l).ne.k) then
         nz(i)=nz(i)+1
         nfo(i,nz(i))=nfo(i,l-1)
         numb(i,nz(i))=l-1
         k=nfo(i,l)
        endif
       enddo
       if(nzz.gt.0) then 
        nz(i)=nz(i)+1
        nfo(i,nz(i))=nfo(i,nzz)
        numb(i,nz(i))=nzz
       endif
c***************Finish preparation***************************
       nq(i)=npom
       nroo=nroo+npom
       write(*,'(a,i3)') 'Done with block ',i
      enddo
      close(unit=9)
c***************Check dimensions*******************************
      k=0
      do i=1,nb
       if(nz(i).eq.0) then
        print*,'no active force constants in block ',i
        stop 4
       endif
       k=k+nq(i)
      enddo
      if(k.lt.nvar) then
       print *,'Total dimension of jacobian = ',k
       print *,'Number of variable force constants = ',nvar
       print *,'System is underdetermined. Increase number of'
       print *,'frequencies or fix more force constants.'
       stop 5
      endif
c**************Diagonalize H and construct q vectors***********
      call hmtx(nb,f,ve)
c*******************Timing***********************************
      if(ve) then
       eti=etime(tarray)
       t1=tarray(1)
       t2=tarray(2)
       tt=t1+t2
       print *
       print *,'  == ETIME ==    User time: ',t1,' seconds'
       print *,'               System time: ',t2,' seconds'
       print *,'                Total time: ',tt,' seconds'
       print *
      endif
c*************Start refinement*******************************
      it=1
      print *,'Iteration : ',it
      print *,'Weighted square deviation : ',f
      write(7,*) 'LSQ iterations + quasi-Newton optimization'
      nvt2=2*nvar
      ff=0.d0
  100 fff=ff
      ff=f
      call jacobian(nb,nrow,nvar)
      call dgesvd('s','s',nrow,nvar,aa,jarow,a,u,jarow,
     $             w,nfc,work,lwork,info)
c**********Calculate vector u'b**************************
      znorm1=0.d0
      do k=1,nvar
       work(k)=0.0d0
       do l=1,nrow
        work(k)=work(k)+u(l,k)*b(l)
       enddo
       z2=work(k)*work(k)
       znorm1=znorm1+z2
      enddo
      fz=sqrt(f*f-znorm1)
      znorm1=sqrt(znorm1)
      print *,'Refinable part of vector b    : ',znorm1
      write(7,*),'Refinable part of vector b    : ',znorm1
      print *,'Unrefinable part of vector b    : ',fz
      write(7,*),'Unrefinable part of vector b    : ',fz
      if(it.eq.1) then
c**********Initial correlation matrix*********************
       if(nrow.gt.nvar) then
        sigma_l=f*f/(nrow-nvar)
       else
        sigma_l=f*f
       endif
       do i=1,nvar
        do j=i,nvar
         r=0.d0
         do k=1,nvar
          p=1.d8*a(k)
          if(abs(p).lt.1.d-18) p=1.d-18
          r=r+w(k,i)*w(k,j)/p/p
         enddo
         opt(i,j)=sigma_l*r
         if(i.eq.j) opt(i,i)=sqrt(opt(i,i))
        enddo
       enddo
       write(8,*) ' Initial correlation matrix'
       do i=1,nvar
        if(fi(ncc(i)).ne.0.d0) then
         p=fi(ncc(i))
        else
         p=1.d0
        endif
        write(8,558) ncc(i),1.0d10*opt(i,i)/p
        write(8,556)(ncc(j),opt(i,j)/opt(i,i)/opt(j,j), 
     .               j=i+1,nvar)
       enddo
  558  format(1x,/,' force constant # ',i3,'.
     .        uncertainty ',e10.3,'%')
c***********truncation parameters************************
       print *,'SVD error=',info
       write(7,*)'Vector (1/d)u''b'
       write(7,555)(work(l)/a(l), l=1,nvar)
  555  format(7(1x,e10.3))
       nsing=0
       nvunc=0
  101  n1=nvunc+1
       if(abs(work(n1)/a(n1)).lt.rlevel) then
        nvunc=n1
        if(nvunc.eq.nvar) goto 102
        goto 101
       endif
       do i=1,nvar
        if(a(i).lt.1.0d-8) then
         print *,'Singular value # ',i,'= ',a(i)
         write(8,*)'Singular value # ',i,'= ',a(i)
         nsing=nsing+1
        endif
       enddo
  102  ntrunc=nvar-nvunc
       print *,'Number of truncated singular values : ',ntrunc
       if(nvunc.eq.0) then
        print *,'Rlevel is too low'
        stop 6
       endif
       nting=ntrunc-nsing
       nving=nvar-nsing
      endif
c************Obtain new set of force constants (TSVD)********
      do i=1,nvar
       x=0.0d0
       do k=1,nvunc
        opt(k,i)=w(k,i)/a(k)
        x=x+opt(k,i)*work(k)
       enddo
       fi(ncc(i))=fi(ncc(i))+x
      enddo
c*************Alternative method (modified L-M)****************
c      aaa=a(nvunc)*a(nvunc)
c      aaa=sqrt(aaa*a(nvunc)*a(nvunc+1))
c      aaa=sqrt(aaa)
c      do i=1,nvar
c       x=0.0d0
c       do k=1,nvar
c        a1=a(k)*a(k)
c        a2=a1*a1
c        a1=a1*a(k)
c        opt(k,i)=w(k,i)*a1/(a2+aaa)
c        x=x+opt(k,i)*work(k)
c       enddo
c       fi(ncc(i))=fi(ncc(i))+x
c      enddo
c***************Diagonalize H matrix************************
      call hmtx(nb,f,ve)
      print *,'Iteration :',it
      print *,'Weighted square deviation  : ',f
      write(7,*)'=========================================='
      write(7,*)'Iteration :',it
      write(7,*),'Weighted square deviation : ',f
      if(ve) then
       write(7,*)'Force constants'
       write(7,554)(fi(i), i=1,nf)
       write(7,*)'Force constant increments'
       write(7,554)(fi(i)-fi0(i), i=1,nf)
      endif
      it=it+1
      change=.false.
      if(znorm1.lt.eps.or.it.gt.maxiter.or.abs(f).lt.eps) then
       change=.true.
       go to 199
      else
       if(abs(f-ff).lt.eps.or.abs(f-fff).lt.eps) then
        e=0.0d0
        do k=1,nvunc
         e=e+abs(work(k)/a(k))
        enddo
        print *,'E = ',e
        if(e.le.nvunc*eps) go to 199
       endif
      endif
      go to 100
c*************output force const. and covariance matrix****
  199 write(7,*)'Force constants'
      write(7,554)(fi(i), i=1,nf)
      write(7,*)'Force constant increments'
      write(7,554)(fi(i)-fi0(i), i=1,nf)
  554 format(10(1x,f7.4))
      if(nrow.gt.nvar) then
       sigma_l=f*f/(nrow-nvar)
      else
       sigma_l=f*f
      endif
      do i=1,nvar
       do j=i,nvar
        r=0.d0
        do k=1,nvunc
         r=r+opt(k,i)*opt(k,j)
        enddo
        g(i,j)=sigma_l*r
        if(i.eq.j) g(i,i)=sqrt(g(i,i))
       enddo
      enddo
      write(7,*) 'Uncertainties and correlation matrix'
      do i=1,nvar
       write(7,557) ncc(i),100*g(i,i)/fi(ncc(i))
       write(7,556)(ncc(j),g(i,j)/g(i,i)/g(j,j), j=i+1,nvar)
      enddo
  556 format(6(1x,i3,1x,f7.4))
  557 format(1x,/,'Force constant # ',i3,'.
     . uncertainty ',f8.2,'%')
      if(change) go to 999
c*******************Timing***********************************
  200 if(ve) then
       eti=etime(tarray)
       t1=tarray(1)
       t2=tarray(2)
       tt=t1+t2
       print *
       print *,'  == ETIME ==    User time: ',t1,' seconds'
       print *,'               System time: ',t2,' seconds'
       print *,'                Total time: ',tt,' seconds'
       print *
      endif
c**************Start quasi-Newton optimtization**************
      e=eee
      es=0.d0
      do k=1,nvunc
       r=work(k)/a(k)
       es=es+r*r
      enddo
      es=es*e      
      do k=nvunc+1,nving
       r=work(k)/a(k)
       es=es+r*r
      enddo
      if(es.lt.eps) goto 100
      print *,'Quasi-Newton optimization started'
      print *,'Objective function : ',es
      iit=0
      call jacobian(nb,nrow,nvar)
      call dgesvd('s','s',nrow,nvar,aa,jarow,a,u,jarow,
     $             w,nfc,work,lwork,info)
c**************Starting Hessian*****************************
      do i=1,nvar
       do j=i,nvar
        p=0.0d0
        do l=nvunc+1,nving
         p=p+w(l,i)*w(l,j)
        enddo
        opt(i,j)=p
        opt(j,i)=p
       enddo
      enddo
      do k=1,nving
       do i=1,nrow
        u(i,k)=u(i,k)/a(k)
       enddo
      enddo
      call jacobian(nb,nrow,nvar)
      call gradient(nrow,nvar,nvunc,nving,e,a)
  210 ff=f
      do j=1,nvar
       fi0(ncc(j))=fi(ncc(j))
       gamma(j)=a(j)
      enddo
      znorm=0.0d0
      do i=1,nvar
       r=0.0d0
       do j=1,nvar
        r=r-opt(i,j)*a(j)
       enddo
       sigma(i)=r
       znorm=znorm+r*r
      enddo
      znorm=sqrt(znorm)
      if(znorm.lt.eps) goto 100
      do i=1,nvar
       sigma(i)=sigma(i)/znorm
      enddo
      x=fmin(a,sigma,f,e,ss,nb,nvar,nrow,nvunc,nving,ve)
      print *,'========================================='
      print *,'Iteration :',it
      print *,'Weighted square deviation : ',f
      print *,'Objective function : ',ss
      write(7,*)'============================================'
      write(7,*)'Iteration :',it
      write(7,*)'Weighted square deviation : ',f
      write(7,*)'Objective function : ',ss
      if(ve) then
       write(7,*)'Force constants'
       write(7,554)(fi(i), i=1,nf)
       write(7,*)'Directional minimum',x
       write(7,*)'-------------------------------------'
       write(7,*)'Search direction'
       write(7,555)(sigma(l), l=1,nvar)
      endif
      it=it+1 
      iit=iit+1
      call jacobian(nb,nrow,nvar)
      if(f.gt.ff) then
       do j=1,nvar
        fi(ncc(j))=fi0(ncc(j))
       enddo
       call hmtx(nb,f,ve)
       goto 100
      else
       if(iit.ge.6.or.abs(x).lt.eps) then
        call hmtx(nb,f,ve)
        goto 100
       endif
      endif
      call gradient(nrow,nvar,nvunc,nving,e,a)
c**************Vector sigma**sigma(1:nvar)*************
c************Vector gamma****gamma(1:nvar)***********
      do i=1,nvar
       sigma(i)=x*sigma(i)
       gamma(i)=a(i)-gamma(i)
      enddo
c************Vector h*gamma***hgamma(1:nvar)********
      do i=1,nvar
       zz=0.0d0
       do j=1,nvar
        zz=zz+opt(i,j)*gamma(j)
       enddo
       hgamma(i)=zz
      enddo
      call updateh(sigma,gamma,hgamma,nvar,opt)
      iit=iit+1
      goto 210       
c*****************Write output and quit**************************
  999 open(unit=9,file=name(1:l_name)//'.fi',status='unknown')
      write(9,99) (fi(i), i=1,nf)
   99 format(10(1x,f7.4))
      close(unit=9)
      stop 1
c  ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 9990 print *
      print *,'Input operation unsuccesful. Check the name and format'
      print *,'of the data file. Don"t input extension of filename !'
      print *,'File : '//name(1:l_name)//'.rein'
      stop 2
c  ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      end

      integer function inopt(ch,optvector)
      implicit real*8(a-h,o-z)
      character*1 ch
      character*4 optvector
      k=0
      do i=1,len(optvector)
       if(ch.eq.optvector(i:i)) k=i
      enddo
      inopt=k
      return
      end

      subroutine readg(cr,l_cr,i,nfq,w,ldw,ncoln,npom,ve)
      implicit real*8(a-h,o-z)
      include 'Sizes'
      character*20 cr
      logical ve
      dimension gg(nint,nint),eigval(nint),w(ldw,ncoln),work(lwork)
      common /hkmatrices/ nr(nbl,ndimz),nc(nbl,ndimz),z(nbl,ndimz),
     .                    numb(nbl,nfc)
c***************Read G-matrix block******************************
       print *,'Reading '//cr(1:l_cr)//'.gmat'
       open(unit=7,file=cr(1:l_cr)//'.gmat',status='old',form
     . ='unformatted')
       read(7) nel
       read(7) (nr(i,l),nc(i,l),z(i,l),l=1,nel)   
       close(unit=7)
c***************Initialize G-matrix******************************
       do k=1,ncoln
        do l=1,ncoln
         gg(k,l)=0.d0
        enddo
       enddo
       do l=1,nel
        gg(nr(i,l),nc(i,l))=z(i,l)
        gg(nc(i,l),nr(i,l))=z(i,l)
       enddo
c****************Diagonalize g matrix***********************
       call dsyev('V','U',ncoln,gg,nint,eigval,work,lwork,info)
       if(ve) then
        print *,'  == READG == '
        print *,'  == DSYEV ==  Error code on return : ',info
        write(*,'(a,f6.0)')
     .          '   == DSYEV ==  Optimal value of lwork : ',work(1)
        print *,'  == DSYEV ==  Current value of lwork : ',lwork
       endif
c****************Calculate scaled eigenvectors***************
       nzero=0
       do k=1,ncoln
        kk=ncoln-k+1
        if(dabs(eigval(kk)).lt.5.0d-5) then
         nzero=nzero+1
         eigval(kk)=0.d0
        endif
        do j=1,ncoln
         w(j,k)=gg(j,kk)*dsqrt(eigval(kk))
        enddo
       enddo
       npom=ncoln-nzero
       if(npom.ne.nfq) then
        print *,'Number of frequencies for',i,' block does'
        print *,'not match the rank of G-matrix = ',npom
        stop 3
       endif
      end

      subroutine jacobian(nb,nrow,nvar)
      implicit real*8(a-h,o-z)
      include 'Sizes'
      parameter (inp=1)
      dimension ww(nint,ndim),el(nint)
      common /qmat/ q(ndim,ndim,nbl),fq(nbl,ndim),sfq(nbl,ndim),
     .        nq(nbl),nz(nbl),nzfq(nbl,ndim),nfo(nbl,ndimz)
     . /jacob/ aa(jarow,nfc),u(jarow,nfc),ncc(nfc)
     . /hkmatrices/ nr(nbl,ndimz),nc(nbl,ndimz),z(nbl,ndimz),
     . numb(nbl,nfc)
c*************Construct jacobian matrix and decompose it*****
      rewind inp
      nrow=0
      do i=1,nb
       read(inp) ii,nrw,ncw,((ww(k,l), l=1,ncw), k=1,nrw)
       ncol=0
       do j=1,nz(i)
        if(j.eq.1) then
	 low=0
        else 
	 low=numb(i,j-1)
        endif
        ncol=ncol+1
  110   if(ncc(ncol).eq.nfo(i,j)) then
         kkk=1
         do l=1,nq(i)
          if(l.ne.nzfq(i,kkk)) then
           nql1=nq(i)-l+1
           call dgemv('N',nrw,nq(i),1.d0,ww,nint,
     .                 q(1,nql1,i),1,0.d0,el,1)
           r=0.d0
           do jj=low+1,numb(i,j)
            nnr=nr(i,jj)
            nnc=nc(i,jj)
            pom=el(nnr)*el(nnc)*z(i,jj)
            r=r+pom
            if(nnr.ne.nnc) r=r+pom
           enddo
c**************Un-weighted Jacobian**********
c           aa(nrow+l,ncol)=r
c********************************************
           aa(nrow+l,ncol)=r/sfq(i,l)
          else
           kkk=kkk+1
           aa(nrow+l,ncol)=0.d0
          endif
         enddo
        else
         do l=nrow+1,nrow+nq(i)
          aa(l,ncol)=0.0d0
         enddo
         ncol=ncol+1
         goto 110
        endif
       enddo
       if(ncol.lt.nvar) then
        do nn=ncol+1,nvar
         do l=nrow+1,nrow+nq(i)
          aa(l,nn)=0.0d0
         enddo
        enddo
       endif
       nrow=nrow+nq(i)
      enddo
      return
      end

      subroutine hkmat(i,k,ldw,ncw,ww,nhk,hk)
      implicit real*8(a-h,o-z)
      include 'Sizes'
c**************Calculate Hk matrices********************
      dimension ww(ldw,ncw),hk(nhk)
      common /hkmatrices/ nr(nbl,ndimz),nc(nbl,ndimz),
     . z(nbl,ndimz),numb(nbl,nfc)
      if(k.eq.1) then
       low=0
      else 
       low=numb(i,k-1)
      endif
      npos=0
      do n=1,ncw
       do m=1,n
        r=0.0d0
        do j=low+1,numb(i,k)
         r=r+ww(nr(i,j),m)*ww(nc(i,j),n)*z(i,j)
         if(nr(i,j).ne.nc(i,j)) r=r+ww(nc(i,j),m)*ww(nr(i,j),n)*z(i,j)
        enddo
        hk(npos+m)=r
       enddo
       npos=npos+n
      enddo
      end

      subroutine diagh(i,inp,eigval,ve)
      implicit real*8(a-h,o-z)
      include 'Sizes'
      logical ve
      dimension ww(nint,nint),h(nkdim),hk(nkdim),work(lwork),
     .eigval(ndim)
      equivalence (hk(1),work(1))
      common /hkmatrices/ nr(nbl,ndimz),nc(nbl,ndimz),z(nbl,ndimz),
     .                    numb(nbl,nfc)
     . /qmat/ q(ndim,ndim,nbl),fq(nbl,ndim),sfq(nbl,ndim),
     .        nq(nbl),nz(nbl),nzfq(nbl,ndim),nfo(nbl,ndimz)
     . /hmat/ h0(nkdim,nbl),b(jarow),fi(nfc),fi0(nfc)
c*************Diagonalization routine**************************
   10 read(inp) iw,nrw,ncw,((ww(k,l), l=1,ncw), k=1,nrw)
      if(iw.ne.i) goto 10
      if(ve) print *,'  == DIAGH ==  Preparing matrix H'
      np=0
      do jj=1,nq(i)
       do ii=1,jj
         h(np+ii)=h0(np+ii,i)
       enddo
       np=np+jj
      enddo
      nhk=nq(i)*(nq(i)+1)/2
      do j=1,nz(i)
       fc=fi(nfo(i,j))
       if(fc.ne.0.d0) then
        call hkmat(i,j,nint,ncw,ww,nhk,hk)
        np=0
        do jj=1,nq(i)
         do ii=1,jj
           h(np+ii)=h(np+ii)+fc*hk(np+ii)
         enddo
         np=np+jj
        enddo
       endif
      enddo
      if(ve) print *,'  == DIAGH ==  Begin diagonalization'
      call dspev('V','U',nq(i),h,eigval,q(1,1,i),ndim,work,info)
      if(ve) then
       print *,'  == DSPEV ==  Error code on return : ',info
       write(*,'(a,f6.0)')
     .         '   == DSPEV ==  Optimal value of lwork : ',work(1)
       print *,'  == DSPEV ==  Current value of lwork : ',lwork
      endif
      end

      subroutine hmtx(nb,fa,ve)
      implicit real*8(a-h,o-z)
      include 'Sizes'
      parameter (inp=1)
      dimension eigval(ndim)
      logical ve
      common /qmat/ q(ndim,ndim,nbl),fq(nbl,ndim),sfq(nbl,ndim),
     .        nq(nbl),nz(nbl),nzfq(nbl,ndim),nfo(nbl,ndimz)
     . /hmat/ h0(nkdim,nbl),b(jarow),fi(nfc),fi0(nfc)
c***************Diagonalization of H matrix*****************
      f=0.0d0
      rewind inp
      nrow=0
      if(ve) print *,'  == HTMX ==  Start diagonalization'
      do i=1,nb
       call diagh(i,inp,eigval,ve)
c*************Construct b-vector**************************
       kk=1
       frob=0.0d0
       do j=1,nq(i)
        nqj1=nq(i)-j+1
        if(eigval(nqj1).lt.0.d0) print *,j,'-th eigenvalue < 0 !'
        if(j.ne.nzfq(i,kk)) then
c************Un-weighted b-vector***************
c          p=fq(i,j)-eigval(nqj1)
c***********************************************
         p=(fq(i,j)-eigval(nqj1))/sfq(i,j)
         b(nrow+j)=p
         frob=frob+p*p
        else
         b(nrow+j)=0.0d0
         kk=kk+1
        endif
       enddo
       f=f+frob
       nrow=nrow+nq(i)
      enddo
      fa=sqrt(f)
      return
      end

      function fmin(a,work,f,e,sx,nb,nvar,nrow,nvunc,nving,ve)
      implicit real*8(a-h,o-z)
      parameter (zero=0.d0,one=1.d0,dwa=2.d0,trzy=3.d0,
     .           trzydwa=3.d0/2.d0,eps=3.d-8,tau=3.81966011d-1)
      logical ve
      dimension work(nvar),a(nvar)
c************Directional search - Davidon's method*****************
      c=zero
      s0=s(zero,work,f,e,nb,nvar,nrow,nvunc,nving,ve)
      g0=zero
      do i=1,nvar
       g0=g0+work(i)*a(i)
      enddo
      if(g0.ge.zero) print *,'fmin warning: g0>0 !!!'
      alpha=one/trzy
   10 sa=s(alpha,work,f,e,nb,nvar,nrow,nvunc,nving,ve)
      call jacobian(nb,nrow,nvar)
      call gradient(nrow,nvar,nvunc,nving,e,a)      
      ga=zero
      do i=1,nvar
       ga=ga+work(i)*a(i)
      enddo
      if(ga.le.zero.and.sa.le.s0) then
       alpha=dwa*alpha
       goto 10
      endif
      gx=1.d0
   20 w=alpha-c
      if(abs(w/alpha).le.eps.or.abs(gx).le.eps) then
       fmin=xx
       return
      endif
      zz=w*(ga+g0)-dwa*(sa-s0)
      wz=w*w/zz
      y2y3=wz*(ga-g0)/dwa-trzydwa*(alpha+c)
      y1y3=wz*(g0*alpha-ga*c)+trzy*alpha*c
      bzyk=y2y3*y2y3-trzy*y1y3
      if(abs(bzyk).gt.eps) then
       bzyk=dsign(one,zz/w)*sqrt(bzyk)
       xx=(bzyk-y2y3)/trzy
      else
       xx=-y2y3/trzy
      endif
      sx=s(xx,work,f,e,nb,nvar,nrow,nvunc,nving,ve)
      call jacobian(nb,nrow,nvar)
      call gradient(nrow,nvar,nvunc,nving,e,a)
      gx=zero
      do i=1,nvar
       gx=gx+work(i)*a(i)
      enddo
      if(gx.ge.zero) then
       sa=sx
       ratio=abs(ga/gx)
       ga=gx
       alpha=xx
      else
       s0=sx
       ratio=abs(g0/gx)
       g0=gx
       c=xx
      endif
      print *,xx,sx,gx
      if(ratio.lt.dwa) goto 30
      goto 20
c************Directional minimum - golden search method*****
   30 print *,'Switching to golden search ...'
      w=tau*(alpha-c)
      t1=c+w
      t2=alpha-w
      f1=s(t1,work,f,e,nb,nvar,nrow,nvunc,nving,ve)
      f2=s(t2,work,f,e,nb,nvar,nrow,nvunc,nving,ve)
   40 if(f1.lt.f2) then
       alpha=t2
       t2=t1
       f2=f1
       w=alpha-c
       t1=c+tau*w
       f1=s(t1,work,f,e,nb,nvar,nrow,nvunc,nving,ve)
       print *,t1,f1
       if(abs(w/alpha).le.eps) then
        fmin=t1
        sx=f1
        return
       endif
       goto 40
      else
       c=t1
       t1=t2
       f1=f2
       w=alpha-c
       t2=alpha-tau*w
       f2=s(t2,work,f,e,nb,nvar,nrow,nvunc,nving,ve)
       print *,t2,f2
       if(abs(w/alpha).le.eps) then
        fmin=t2
        sx=f2
        return
       endif
       goto 40
      endif
      end

      function s(x,d,f,e,nb,nvar,nrow,nvunc,nving,ve)
      implicit real*8(a-h,o-z)
      include 'Sizes'
      logical ve
      dimension d(nvar)
      common /jacob/ aa(jarow,nfc),u(jarow,nfc),ncc(nfc)
     1/hmat/ h0(nkdim,nbl),b(jarow),fi(nfc),fi0(nfc)
c*************Find value of s(fi+x*d)************************
      do i=1,nvar
       fi(ncc(i))=fi0(ncc(i))+x*d(i)
      enddo
      call hmtx(nb,f,ve)
      ss=0.0d0
      do k=1,nvunc
       p=0.0d0
       do i=1,nrow
        p=p+u(i,k)*b(i)
       enddo
       ss=ss+p*p
      enddo
      ss=ss*e
      do k=nvunc+1,nving
       p=0.0d0
       do i=1,nrow
        p=p+u(i,k)*b(i)
       enddo
       ss=ss+p*p
      enddo
      s=ss
      return
      end

      subroutine gradient(nrow,nvar,nvunc,nving,e,gr)
      implicit real*8(a-h,o-z)
      include 'Sizes'
      parameter(dwa=2.0d0)
      dimension gr(nvar+nving)
      common /jacob/ aa(jarow,nfc),u(jarow,nfc),ncc(nfc)
     1/hmat/ h0(nkdim,nbl),b(jarow),fi(nfc),fi0(nfc)
c************Calculate gradient of penalty function*******
      do k=1,nving
       gg=0.0d0
       do i=1,nrow
        gg=gg+u(i,k)*b(i)
       enddo
       gr(nvar+k)=gg
      enddo
      do j=1,nvar
       gg=0.0d0
       do k=1,nvunc
        p=0.0d0
        do i=1,nrow
         p=p+u(i,k)*aa(i,j)
        enddo
        gg=gg+p*gr(nvar+k)
       enddo
       gg=e*gg
       do k=nvunc+1,nving
        p=0.0d0
        do i=1,nrow
         p=p+u(i,k)*aa(i,j)
        enddo
        gg=gg+p*gr(nvar+k)
       enddo
       gr(j)=-dwa*gg
      enddo
      return
      end

      subroutine updateh(sigma,gamma,hgamma,nvar,opt)
      implicit real*8(a-h,o-z)
      dimension sigma(nvar),gamma(nvar),hgamma(nvar),
     .          opt(nvar,nvar)
c************Quasi-Hessian update********************
      skal1=0.d0
      skal2=0.d0
      do i=1,nvar
       skal1=skal1+sigma(i)*gamma(i)
       skal2=skal2+gamma(i)*hgamma(i)
      enddo
      coef=(1.d0+skal2/skal1)/skal1
      do i=1,nvar
c************Davidon-Fletcher-Powell***************
c       opt(i,i)=opt(i,i)+sigma(i)*sigma(i)/skal
c       opt(i,i)=opt(i,i)-hgamma(i)*hgamma(i)/skal2

c*******Broyden-Fletcher-Goldfarb-Shanno************
       opt(i,i)=opt(i,i)+coef*sigma(i)*sigma(i)
       opt(i,i)=opt(i,i)-2.d0*sigma(i)*hgamma(i)/skal1

       do j=i+1,nvar
c************Davidon-Fletcher-Powell***************
c        opt(i,j)=opt(i,j)+sigma(i)*sigma(j)/skal
c        opt(i,j)=opt(i,j)-hgamma(i)*hgamma(j)/skal2

c*******Broyden-Fletcher-Goldfarb-Shanno************
        opt(i,j)=opt(i,j)+coef*sigma(i)*sigma(j)
        opt(i,j)=opt(i,j)
     . -(sigma(i)*hgamma(j)+hgamma(i)*sigma(j))/skal1

        opt(j,i)=opt(i,j)
       enddo
      enddo
      return
      end
